#*************************************************************************
#*									 *
#*     Copyright c 1998 The board of regents of the University of 	 *
#*                      Florida.  All rights reserved.			 *
#*     This subroutine may not be used outside of the FLOO* computer	 *
#*     programs without the prior written consent of the University of 	 *
#*     Florida.								 *
#*									 *
#*************************************************************************/
#*   Damage.tcl                Version 1.2     */
#*   Last Modification : 06/11/98 10:19:45  */

proc Max {d1 d2} {
    if { $d1 > $d2 } {
	return $d1
    } else {
	return $d2
    }
}

#this procedure takes the file names of two files created by
#a monte-carlo program (we use UTMarlowe), and builds the
#defect profiles appropriate for initial conditions for
#diffusion
proc CreateDamage {DamageFile ImplantFile ImpName temp} {
    #create the equilibrium initial condition
    InitDefect $temp
    InitDopantPairs $temp

    #set the temperature and get the equilibrium concentrations
    SetTemp $temp
    set csi [pdbGetDouble Si Interstitial Cstar]
    set csv [pdbGetDouble Si Vacancy Cstar]

    #read in the Damage profile and smooth it
    profile name=Damage infile=$DamageFile
    set Par [sigma Damage silicon 1.0e16]
    BuildProfile SmoothDamage $Par

    #read in the plus one profile and smooth it
    profile name=plusone infile=$ImplantFile
    set Par [sigma plusone silicon 1.0e15]
    BuildProfile PlusOne $Par

    #create the defect profiles
    sel z=Interstitial+SmoothDamage name=Interstitial store
    sel z=Vacancy+SmoothDamage name=Vacancy store

    #add in the implanted contribution
    sel z=$ImpName+PlusOne name=$ImpName store

    #now comes the hard part - get all present dopants initialized
    set sol [sel list]

    foreach dop "[dopant list]" {
	if {  [lsearch $sol $dop] != -1 } {
	    sel z=1.0 name=den

	    #compute the substitutional concentration
	    if {[pdbIsAvailable Si $dop Interstitial Binding]} {
		set kp [pdbGetDouble Si $dop Interstitial Binding]
		sel z=den+$kp*Interstitial name=den
	    }
	    if {[pdbIsAvailable Si $dop Vacancy Binding]} {
		set kp [pdbGetDouble Si $dop Vacancy Binding]
		sel z=den+$kp*Vacancy name=den
	    }
	    sel z = "$dop/den" name=${dop}Sub store

	    if {[pdbIsAvailable Si $dop Interstitial Binding]} {
		set kp [pdbGetDouble Si $dop Interstitial Binding]
		sel z = "${dop}Sub*$kp*Interstitial" name=${dop}Interstitial store
		sel z = Interstitial-${dop}Interstitial name=Interstitial store
		tclsel z = {[Max $Interstitial $csi]} name=Interstitial
	    }
	    if {[pdbIsAvailable Si $dop Vacancy Binding]} {
		set kp [pdbGetDouble Si $dop Vacancy Binding]
		sel z = "${dop}Sub*$kp*Vacancy" name=${dop}Vacancy store
		sel z = Vacancy-${dop}Vacancy name=Vacancy store
		tclsel z = {[Max $Vacancy $csv]} name=Vacancy
	    }
	}
    }

}
