proc BarrierConductivity { BarrierHeight BarrierThickness {Beta 1.0} } {

    # Importing fundamental constants
    set q [pdbGetDouble FundamentalConstants q]         ;# [C]
    set h [pdbGetDouble FundamentalConstants h]         ;# [eVs]
    set hbar [pdbGetDouble FundamentalConstants hbar]   ;# [eVs]
    set m_e [pdbGetDouble FundamentalConstants m]       ;# electron mass, [kg]

    # Convert units
    set hbar_J [expr $hbar*$q] ;# hbar [Js]
    set h_J [expr $h*$q]       ;# h [Js]
    set phi_J [expr $BarrierHeight*$q] ;# barrier height in [J]

    # Calculating parameters
    set lambda_m [expr $hbar_J/($Beta*2.*sqrt(2.*$m_e*$phi_J))]     ;# lambda [m]
    set lambda [expr $lambda_m*100.]                         ;# lambda [cm]
    set A_Sm [expr 3./2. * sqrt(2.*$m_e*$phi_J) / $Beta * ($q/$h_J)*($q/$h_J)] ;# A [S/m]
    set A [expr $A_Sm / 100.]                                ;# A [S/cm]

    # conductivity calculation
    set Conductivity "($A *exp( -$BarrierThickness/$lambda ))"

	return $Conductivity
}

proc SmallBarrierConductivity { BarrierHeight BarrierThickness } {

    # Importing fundamental constants
    set q [pdbGetDouble FundamentalConstants q]         ;# [C]
    set h [pdbGetDouble FundamentalConstants h]         ;# [eVs]
    set hbar [pdbGetDouble FundamentalConstants hbar]   ;# [eVs]
    set m_e [pdbGetDouble FundamentalConstants m]       ;# electron mass, [kg]

    # Convert units
    set hbar_J [expr $hbar*$q] ;# hbar [Js]
    set h_J [expr $h*$q]       ;# h [Js]
    set phi_J [expr $BarrierHeight*$q] ;# barrier height in [J]
    
    set Beta 0.95
    set Beta 0.99

    # Calculating parameters
    set lambda_m [expr $hbar_J/($Beta*2.*sqrt(2.*$m_e*$phi_J))]     ;# lambda [m]
    set lambda [expr $lambda_m*100.]                         ;# lambda [cm]
    set paper_A_Sm [ expr 4*acos(-1.0) * $Beta * $BarrierThickness / $h_J * sqrt( 2*$m_e ) ] ;# A [S/m]
    set paper_J0_Sm [ expr $q/(2.*acos(-1.0)*$h_J*$BarrierThickness*$Beta*$Beta) ] ;# A [S/m]
    set A_Sm [expr 3./2. * sqrt(2.*$m_e*$phi_J) * ($q/$h_J)*($q/$h_J)] ;# A [S/m]
    set A [expr $A_Sm / 100. / $Beta]                                ;# A [S/cm]

    # puts ""
    # puts $paper_J0_Sm 
    # puts $paper_A_Sm 
    # puts $A_Sm
    # puts [ expr 3./2. * $paper_J0_Sm*$paper_A_Sm*$q*sqrt($phi_J)/2. ]
    # puts [ expr 3./2. * $paper_J0_Sm*$q*($paper_A_Sm*sqrt($phi_J)/2. - 1) ]

    set A [ expr 3./2. * $paper_J0_Sm*$q*($paper_A_Sm*sqrt($phi_J)/2. - 1) / 100 ]

    # conductivity calculation
    set Conductivity "($A *exp( -$BarrierThickness/$lambda ))"

	return $Conductivity
}


