namespace eval AlGaN {
#set low field electron mobility via analytical expression from Farahmand for low field mobility
proc ElecMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=Temp"
  lappend usage "The temperature (T) defaults to an alagator expression of Temp"
  set returns "AlGaN electron mobility alagator string"
  set description {Computes the temperature dependent electron mobility for AlGaN}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T ]
  if {$T == ""} { set T Temp }
  set T ($T)

  set Amumin 312.1
  set Amumax 1401.3
  set Alowalpha 0.74
  set Abeta1 -6.51
  set Abeta2 -2.31
  set Abeta3 7.07
  set Abeta4 -0.86
  set ANref 1e17
  
  #build equation for low field mobility
  set Aseg1 "$Amumin*(exp(log($T/300)*($Abeta1)))"
  set Aseg2 "($Amumax-$Amumin)*(exp(log($T/300)*($Abeta2)))"
  set Aseg3 "$ANref*(exp(log($T/300)*($Abeta3)))"
  set Aseg4 "abs((Doping+1)/$Aseg3)"
  set Aseg5 "$Alowalpha*(exp(log($T/300)*($Abeta4)))"
  set Aseg6 "exp(log($Aseg4)*($Aseg5))"
  set Aseg7 "1+$Aseg6"
  
  #set electron mobility via analytical expression from Farahmand for high field mobility U=0
  set Alowfldmob "($Aseg1+(($Aseg2)/($Aseg7)))"
  set Ahighalpha 6.9502
  set An1 7.8138
  set An2 0.7897
  set AEcmob 245579.4
  set Avsat 2.02e7
  set AEfield "(sqrt(dot(DevPsi,DevPsi)+1.0))"
  set AEfield_EcRatio "($AEfield)/($AEcmob)"
  
  set AEn1 "exp(log($AEfield_EcRatio)*($An1-1))"
  set AEn2 "exp(log($AEfield_EcRatio)*($An2))"
  
  set Anum "($Alowfldmob+($Avsat*($AEn1)/($AEcmob)))"
  set Aden "(1.0+($Ahighalpha*($AEn2))+($AEn1*$AEfield_EcRatio))"
  
  #-------------constant mobilities-----------------------------------
  return 213.3
}
#set hole mobility as constant
proc HoleMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns "AlGaN hole mobility alagator string"
  set description {Gives a rough AlGaN hole mobility}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  return 0.2
}
}

