# The driving force is the only argument

proc B2BE1 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name F=E mat=Silicon"
  lappend usage "The driving force/field (F) is optional and defaults to E."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "Simple band to band tunneling alagator string"
  set description {Simple linear band to band tunneling model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set F [__GetValue $args F ]
  if {$F == ""} { set F E }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  if {$Material != "Silicon"} { puts "$name Error: Materials other than silicon not currently supported"; exit; }
  set A 1.1e27 ;# 1/(cm^2*s*V)
  set B 21.3e6 ;# V/cm
  return ($A*$F*exp(-$B/$F))
}

proc B2BE1_5 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name F=E mat=Silicon"
  lappend usage "The driving force/field (F) is optional and defaults to E."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "Simple band to band tunneling alagator string"
  set description {Simple $x^{1.5}$ power band to band tunneling model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set F [__GetValue $args F ]
  if {$F == ""} { set F E }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  if {$Material != "Silicon"} { puts "$name Error: Materials other than silicon not currently supported"; exit; }

  set A 1.9e24 ;# 1/(cm^3/2*s*V^3/2)
  set B 21.9e6 ;# V/cm
  return ($A*($F)^1.5*exp(-$B/$F))
}

proc B2BE2 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name F=E mat=Silicon"
  lappend usage "The driving force/field (F) is optional and defaults to E."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "Simple band to band tunneling alagator string"
  set description {Simple quadratic band to band tunneling model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set F [__GetValue $args F ]
  if {$F == ""} { set F E }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  if {$Material != "Silicon"} { puts "$name Error: Materials other than silicon not currently supported"; exit; }

  set A 3.4e21 ;# 1/(cm*s*V^2)
  set B 21.6e6 ;# V/cm
  return ($A*($F)^2*exp(-$B/$F))
}


proc B2BE1Taylor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name F=E mat=Silicon"
  lappend usage "The driving force/field (F) is optional and defaults to E."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "Simple band to band tunneling alagator string"
  set description {Simple linear band to band tunneling model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set F [__GetValue $args F ]
  if {$F == ""} { set F E }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  if {$Material != "Silicon"} { puts "$name Error: Materials other than silicon not currently supported"; exit; }
  set A 1.1e27 ;# 1/(cm*s*V)
  set B 21.3e6 ;# V/cm
  # return ($A*$F*(1+(-$B/$F)+(-$B/$F)*(-$B/$F)/2)
  # linear expansion
  set v ($A*$F-[expr $A*$B])
  # quadratic expansion
  set v ($A*$F-[expr $A*$B]+[expr 0.5*$A*$B*$B]/$F-[expr (1./2./3.)*$A*$B*$B*$B]/$F/$F+[expr (1./2./3./4.)*$A*$B**4]/$F/$F/$F-[expr (1./2./3./4./5.)*$A*$B**4]/$F/$F/$F/$F)
  return ($v)
}

proc Factorial {x} {
   set i 1; set product 1
   while {$i <= $x} {
      set product [expr $product * $i]
      incr i
   }
   return $product
}

proc TaylorTest {} {
set F 1
set ans [expr [B2BE1 F=$F\e6]]
puts $ans
set A 1.1e33 ;# 1/(cm*s*V)
set B 21.3 ;# V/cm
set Taylor 0
for { set i 0 } { $i < 100 } { incr i } {
  set Taylor [expr $Taylor+$A*$F*(-$B/$F)**$i / [Factorial $i]]
  puts "$i [expr $Taylor] [expr $Taylor/$ans]"
}
}
# TaylorTest

