proc FEQFOhmic args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? name=contact_name Doping=Doping Hole=Hole Elec=Elec ?current?"
  lappend usage "Temperature is required in Kelvin"
  lappend usage "Contact name (name) is set in the pdb with the equations"
  lappend usage "The Doping concentration (Doping) is calculated with the default material Hole model unless otherwise specified"
  lappend usage "The Hole concentration (Hole) is calculated with the default material Hole model unless otherwise specified"
  lappend usage "The Electron concentration (Elec) is calculated with the default material Elec model unless otherwise specified"
  lappend usage "A current source can be turned on with the `current' flag"
  set returns "nothing"
  set description {Convienence wrapper for Finite Element Quasi Fermi (FEQF) simulation.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set contact_name [__GetValue $args name "$name Error: No contact name specified"]
  set T [__GetValue $args T "$name Error: Temperature (T) specified"]
  set mater [contact name= $contact_name get_mater]
  set Doping [__GetValue $args Doping]
  if { $Doping == "" } { 
    set Doping Doping 
  }
  set Hole [__GetValue $args Hole]
  if { $Hole == "" } { 
    set Nv [$mater\::Nv T=$T] ;# 1/cm^3
    set Eg ([$mater\::BandGap T=$T])
    set Hole [ValFermiDirac Ev=[Eval Eg=$Eg] Nv=$Nv T=$T] 
  }
  set Elec [__GetValue $args Elec]
  if { $Elec == "" } { 
    set Nc [$mater\::Nc T=$T] ;# 1/cm^3
    set Elec [CondFermiDirac Ec=[Econd] Nc=$Nc T=$T] 
  }
  set voltage [expr ![__Contains $args current]]
  set hole_current [expr [contact name=$contact_name avgval= $Doping] < 0]
  pdbSet $contact_name Qfn Flux 1
  pdbSet $contact_name Qfp Flux 1
  pdbSet $contact_name DevPsi Flux 1
  pdbSet $contact_name Qfn Fixed 1
  pdbSet $contact_name Qfp Fixed 1
  pdbSet $contact_name DevPsi Fixed 1
  set q 1.602176634e-19
  pdbSet $contact_name Qfn Flux.Scale $q
  pdbSet $contact_name Qfp Flux.Scale $q
  if { $voltage } {
    pdbSet $contact_name Qfn Equation "Qfn-$contact_name"
    pdbSet $contact_name Qfp Equation "Qfp-$contact_name"
  } else {
    set scal [expr [contact name=$contact_name size]*$q]
    if { $hole_current } {
      pdbSet $contact_name Qfn Equation "Qfn-Qfp"
      pdbSet $contact_name Qfp Fixed 0
      pdbSet $contact_name Qfp Equation "Qfp-($contact_name/$scal)"
    } else {
      pdbSet $contact_name Qfn Fixed 0
      pdbSet $contact_name Qfn Equation "Qfn-($contact_name/$scal)"
      pdbSet $contact_name Qfp Equation "Qfp-Qfn"
    }
  }
  pdbSet $contact_name DevPsi Equation "($Doping) - ($Elec) + ($Hole)"
  pdbSet $contact_name Equation "1.619e-19 * (Flux_Hole - Flux_Elec)"
  pdbSet $contact_name Qfp DampValue 0.025
  pdbSet $contact_name Qfn DampValue 0.025
  pdbSet $contact_name DevPsi DampValue 0.025
  # Used in density gradient
  # Correction potential should be 0 at an ohmic contact
  pdbSet $contact_name Lambda Equation "Lambda"
}

proc SGOhmicCurrent args {
  set name [lindex [info level 0] 0]
  lappend usage "$name name=contact_name Elec=Elec Hole=Hole"
  lappend usage "Contact name (name) is set in the pdb with the equations"
  lappend usage "The electron quasi-Fermi (Elec) is assumed to be Elec"
  lappend usage "The hole quasi-Fermi (Hole) is assumed to be Hole"
  lappend usage "The Electron concentration (Elec) is calculated with the default material Elec model unless otherwise specified"
  set returns "SG Ohmic Contact Current"
  set description {Convienence wrapper for Finite Element Quasi Fermi (FEQF) simulation.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set contact_name [__GetValue $args name "$name Error: No contact name specified"]
  set Elec [__GetValue $args Elec]
  if { $Elec == "" } { 
    set Elec Elec 
  }
  set Hole [__GetValue $args Hole]
  if { $Hole == "" } { 
    set Hole Hole 
  }
  return [expr [contact name=$contact_name sol=$Hole flux]-[contact name=$contact_name sol=$Elec flux]]
}

proc FEQFOhmicCurrent args {
  set name [lindex [info level 0] 0]
  lappend usage "$name name=?contact_name? Qfn=Qfn Qfp=Qfp"
  lappend usage "Contact name (name) is set in the pdb with the equations"
  lappend usage "The electron quasi-Fermi (Qfn) is assumed to be Qfn"
  lappend usage "The hole quasi-Fermi (Qfp) is assumed to be Qfp"
  lappend usage "The Electron concentration (Elec) is calculated with the default material Elec model unless otherwise specified"
  set returns "FEQF Ohmic Contact Current"
  set description {Convienence wrapper for Finite Element Quasi Fermi (FEQF) simulation.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set contact_name [__GetValue $args name "$name Error: No contact name specified"]
  set Qfn [__GetValue $args Qfn]
  if { $Qfn == "" } { 
    set Qfn Qfn 
  }
  set Qfp [__GetValue $args Qfp]
  if { $Qfp == "" } { 
    set Qfp Qfp 
  }
  return [expr [contact name=$contact_name sol=$Qfp flux]-[contact name=$contact_name sol=$Qfn flux]]
}

proc SGOhmic args {
  set name [lindex [info level 0] 0]
  lappend usage "$name name=contact_name T=? ni=? Hole=Hole Elec=Elec"
  lappend usage "Contact name (name) is set in the pdb with the equations"
  lappend usage "The temperature T is in units of Kelvin, this can be a value or an alagator expression"
  lappend usage "The hole concentration (Hole) is calculated with the Hole field unles otherwise specified"
  lappend usage "The Elec concentration (Elec) is calculated with the Elec field unles otherwise specified"
  set returns "nothing"
  set description {Convienence wrapper for SG ohmic contacts.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set ni [__GetValue $args ni "$name Error: No intrensic concentration (ni) specified"]
  set contact_name [__GetValue $args name "$name Error: No contact name specified"]
  set Hole [__GetValue $args Hole]
  if { $Hole == "" } { set Hole Hole }
  set Elec [__GetValue $args Elec]
  if { $Elec == "" } { set Elec Elec }

  set q 1.602176634e-19
  set k 1.38066e-23
  set Vt "([expr $k/$q]*$T)"

  pdbSet $contact_name $Elec  Fixed 1
  pdbSet $contact_name $Hole  Fixed 1
  pdbSet $contact_name DevPsi Fixed 1
  pdbSet $contact_name DevPsi Equation "Doping - ($Elec) + ($Hole)"
  pdbSet $contact_name $Hole  Equation "DevPsi + $Vt*log((($Hole)+1)/$ni) - $contact_name"
  pdbSet $contact_name $Elec  Equation "DevPsi - $Vt*log((($Elec)+1)/$ni) - $contact_name"
  pdbSet $contact_name $Elec  Flux.Scale $q
  pdbSet $contact_name $Hole  Flux.Scale $q

}

