

## Full Density Gradient Correction:
proc DGEquation args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? Lambda=Lambda DevPsi=DevPsi gamma=1.0"
  lappend usage "Temperature (T) needs to be specified in Kelvin"
  lappend usage "Density gradient correction potential potential is assumed to be Lambda"
  lappend usage "Electrostatic potential is assumed to be DevPsi"
  lappend usage "The fitting parameter gamma is assumed to be 1.0"
  set returns "alagator string of density gradient equation"
  set description {Convienence wrapper for gate contacts.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set DevPsi [__GetValue $args DevPsi]
  if { $DevPsi == "" } { set DevPsi DevPsi }
  set Lambda [__GetValue $args Lambda]
  if { $Lambda == "" } { set Lambda Lambda }
  set gamma [__GetValue $args gamma]
  if { $gamma == "" } { set gamma 1.0 }
  set T [__GetValue $args T "$name Error: No Temperature (T) specified"]
  set q  1.619e-19
  set k 1.38066e-23
  set Vt "([expr $k/$q]*$T)"
  set hbar  6.582119569e-16
  set m0 9.10938356e-31 ;#kg
  # set DGConst  [expr $gamma*1e4*$q*($hbar)**2./12./$m0/$Vt]
  set DGConst  "[expr $gamma*1e4*$q*($hbar)**2./12./$m0/($k/$q)]/$T"
  return "$Lambda - $DGConst * grad($Lambda+$DevPsi) + $DGConst * dot( ($Lambda+$DevPsi) , ($Lambda+$DevPsi) ) / (2*$Vt)"
}

proc __DGEquation_simp { Lambda DevPsi Vt } {
  set q  1.619e-19
  set hbar  6.582119569e-16
  set m0 9.10938356e-31 ;#kg
  set gamma 0.3
  set gamma 1.0
  set DGConst  [expr $gamma*1e4*$q*($hbar)**2./12./$m0/$Vt]
  return "$Lambda - $DGConst * grad($Lambda+$DevPsi)"
}

proc __DGCarrier { Lambda gamma Vt } {
  return $Lambda*$gamma/$Vt"
}

proc DGInvMass args {
  set name [lindex [info level 0] 0]
  lappend usage "$name d=d mx=? my=? mz=?"
  lappend usage "Distance from the oxide interface (d) is assumed to have the alagator field d"
  lappend usage "Effective mass in the x direction needs to be specified with mx"
  lappend usage "Effective mass in the y direction needs to be specified with my"
  lappend usage "Effective mass in the z direction needs to be specified with mz"
  set returns "alagator string of density gradient effective mass for a band"
  set description {Convienence wrapper for gate contacts.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set d [__GetValue $args d]
  if { $d == "" } { set d d }
  set mx [__GetValue $args mx "$name Error: No mx effective mass specified"]
  set my [__GetValue $args my "$name Error: No my effective mass specified"]
  set mz [__GetValue $args mz "$name Error: No mz effective mass specified"]

  sel name=__dx z=((dot($d,$d)<1e-10)?0:(dot($d,x)/dot($d,$d)))
  sel name=__dy z=((dot($d,$d)<1e-10)?0:(dot($d,y)/dot($d,$d)))
  sel name=__dz z=((dot($d,$d)<1e-10)?0:(dot($d,z)/dot($d,$d)))
  set scal sqrt(__dx*__dx+__dy*__dy+__dz*__dz) 
  sel name=__dx z=(($scal<1e-10)?0:(abs(__dx)/$scal)) 
  sel name=__dy z=(($scal<1e-10)?0:(abs(__dy)/$scal)) 
  sel name=__dz z=(($scal<1e-10)?0:(abs(__dz)/$scal)) 
  return (__dx/($mx)+__dy/($my)+__dz/($mz))
}



