proc Auger args {
  set name [lindex [info level 0] 0]
  lappend usage "$name ni= Elec=Elec Hole=Hole mat=Silicon"
  lappend usage "The intrensic carrier concentration (ni) is in cm$^{-3}$."
  lappend usage "Hole concentration (Hole) is optional and defaults to Hole."
  lappend usage "Electron concentration (Elec) is optional and defaults to Elec."
  set returns "Auger recombination alagator string"
  set description {Auger recombination}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set ni [__GetValue $args ni "$name Error: No intrensic carrier concentration (ni) given"]
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  if {$Material != "Silicon"} { puts "$name Error: Materials other than silicon not currently supported"; exit; }

  set A_Ae 0.67e-31
  set A_Ah 0.72e-31
  set B_Ae 2.45e-31
  set B_Ah 4.50e-33
  set C_Ae 2.2e-32
  set C_Ah 2.63e-32
  set H_e 3.46667
  set H_h 8.25688
  set N0 1e18
  set T 300.0
  set CnT "(($A_Ae+$B_Ae*($T/$T)-$C_Ae*(($T/$T)^2))*(1+$H_e*exp(-$Elec/$N0)))"
  set CpT "(($A_Ah+$B_Ah*($T/$T)+$C_Ah*(($T/$T)^2))*(1+$H_h*exp(-$Hole/$N0)))"
  return "($CnT*$Elec+$CpT*$Hole)*($Elec*$Hole-$ni*$ni)"
}


