#SRH w doping dependent terms in the lifetimes
proc SRH args {
  set name [lindex [info level 0] 0]
  lappend usage "$name ni= Elec=Elec Hole=Hole Accept=Accept Donor=Donor mat=Silicon"
  lappend usage "The intrensic carrier concentration (ni) is in cm$^{-3}$."
  lappend usage "The donor concentration (Donor) is optional and defaults to Donor."
  lappend usage "The acceptor concentration (Accept) is optional and defaults to Accept."
  lappend usage "Hole concentration (Hole) is optional and defaults to Hole."
  lappend usage "Electron concentration (Elec) is optional and defaults to Elec."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "Shockley–Read–Hall recombination alagator string"
  set description {Shockley–Read–Hall recombination.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set ni [__GetValue $args ni "$name Error: No intrensic carrier concentration (ni) given"]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  if {$Material == "Silicon"} { 
    set tau_max_e 3.0e-5
    set tau_max_h 1.0e-5
    set Nref 1.0e17
  }
  if {$Material == "Germanium"} { 
    set tau_max_e 4.0e-5
    set tau_max_h 4.0e-5
    set Nref 1.0e14
  }

  # Doping Dependence
  set tau_n "$tau_max_e/(1+((($Accept+$Donor)/($Nref))))"
  set tau_p "$tau_max_h/(1+((($Accept+$Donor)/($Nref))))"
  set tau_n "$tau_max_e"
  set tau_p "$tau_max_h"
  return "(($Elec*$Hole)-($ni*$ni))/($tau_p*($Elec+$ni)+$tau_n*($Hole+$ni))"
}



