 
proc QFElecContinuity args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Qfn=Qfn Elec=Elec Emob=Emob ddtElec=Elec G=0 R=0"
  set returns "Continuity equation alagator string"
  set description {$\partial Elec / \partial t - \mu_n n \nabla \Phi_n - R + G$}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Qfn [__GetValue $args Qfn ]
  if {$Qfn == ""} { set Qfn Qfn }
  set Emob [__GetValue $args Emob ]
  if {$Emob == ""} { set Emob Emob }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set ddtElec [__GetValue $args ddtElec ]
  if {$ddtElec == ""} { set ddtElec Elec }
  set G [__GetValue $args G ]
  if {$G == ""} { set G 0 }
  set R [__GetValue $args R ]
  if {$R == ""} { set R 0 }

  return "ddt($ddtElec) + ($Emob) * ($Elec) * grad($Qfn) + ($R) - ($G)"
}

proc QFHoleContinuity args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Qfp=Qfp Hole=Hole Hmob=Hmob ddtHole=Hole G=0 R=0"
  set returns "Continuity equation alagator string"
  set description {$\partial Hole / \partial t + \mu_p p \nabla \Phi_p - R + G$}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Qfp [__GetValue $args Qfp ]
  if {$Qfp == ""} { set Qfp Qfp }
  set Hmob [__GetValue $args Hmob ]
  if {$Hmob == ""} { set Hmob Hmob }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  set ddtHole [__GetValue $args ddtHole ]
  if {$ddtHole == ""} { set ddtHole Hole }
  set G [__GetValue $args G ]
  if {$G == ""} { set G 0 }
  set R [__GetValue $args R ]
  if {$R == ""} { set R 0 }

  return "ddt($ddtHole) - ($Hmob) * ($Hole) * grad($Qfp) + ($R) - ($G)"
}

proc SGElecContinuity args {
  set name [lindex [info level 0] 0]
  lappend usage "$name DevPsi=DevPsi Elec=Elec Emob=Emob G=0 R=0"
  set returns "SG Continuity equation alagator string"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set DevPsi [__GetValue $args DevPsi ]
  if {$DevPsi == ""} { set DevPsi DevPsi }
  set Emob [__GetValue $args Emob ]
  if {$Emob == ""} { set Emob Emob }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set G [__GetValue $args G ]
  if {$G == ""} { set G 0 }
  set R [__GetValue $args R ]
  if {$R == ""} { set R 0 }
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  return "ddt($Elec) + ($Emob) * $Vt * sgrad(($Elec), ($DevPsi)/$Vt) + ($R) - ($G)"
}

proc SGHoleContinuity args {
  set name [lindex [info level 0] 0]
  lappend usage "$name DevPsi=DevPsi Hole=Hole Hmob=Hmob G=0 R=0"
  set returns "SG Continuity equation alagator string"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set DevPsi [__GetValue $args DevPsi ]
  if {$DevPsi == ""} { set DevPsi DevPsi }
  set Hmob [__GetValue $args Hmob ]
  if {$Hmob == ""} { set Hmob Hmob }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  set G [__GetValue $args G ]
  if {$G == ""} { set G 0 }
  set R [__GetValue $args R ]
  if {$R == ""} { set R 0 }
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  return "ddt($Hole) + ($Hmob) * $Vt * sgrad(($Hole), -($DevPsi)/$Vt) + ($R) - ($G)"
}

