namespace eval Germanium {

proc Nc args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= shear=0"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Shear strain (s).}
  set returns {alagator string}
  set description {Temperature dependent electron density of states model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set shear [__GetValue $args shear]
  if { $shear == "" } { set shear 0 } 
  set T [__GetValue $args T "Temperature parameter (T) in Kelvin required"]
  
  set q 1.602e-19
  set k 1.38066e-23
  set Vt ([expr {$k/$q}]*$T)
  set small 1.0e-10
  set pi [ expr acos(-1.0) ]
  set m0 9.10938356e-31 ;#kg
  set hbar 1.05457e-34 ;#J/s
  set mn [ expr 4.**(2./3.) * (1.59*0.0815*0.0815)**(1./3.) * $m0 ]

  return (($T)^(1.5)*[ expr 2.* (  ( $mn*$k )/( 2.*$pi*$hbar**2. ) )**1.5 /100.**3. ]) ;# 1/cm^3
  
}

# for { set X -0.05 } { $X < 0.0505 } { set X [expr $X+0.001] } {
#   puts [PC [aexpr [Nc T=300 shear= $X]] [aexpr [Nc T=300]]]
# }

}
