
namespace eval Germanium {

proc Eqns args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=?"
  lappend usage "Temperature (T) is required in Kelvin."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]

  set Emob [lindex [Germanium::Masetti T=$T ] 0]
  set Hmob [lindex [Germanium::Masetti T=$T ] 1]
  solution Germanium name=Emob add const solve val= $Emob
  solution Germanium name=Hmob add const solve val= $Hmob
  # intrensic carrier concentrations and band edge DOS
  set Nc [Germanium::Nc T=$T] ;# 1/cm^3
  set Nv [Germanium::Nv T=$T] ;# 1/cm^3
  solution Germanium add name=Eg0 const solve val= ([Germanium::BandGap T=$T])
  solution Germanium add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg0 T=$T]

  # Recombination
  solution Germanium add name= SRH const solve val= [SRH ni=ni mat=Germanium]

  solution Germanium add name= R const solve val= (SRH)
  solution Germanium add name= G const solve val= 0

  # Electron Equations
  solution Germanium add name=Econd const solve val= [Econd Eea= 4.0]
  solution Germanium add name=Elec   const solve val= [CondFermiDirac Ec=Econd Nc=$Nc T=$T]
  sel z= 0 name= Bx
  sel z= 0 name= By
  sel z= 0 name= Bz
  # put in the tesla to Gauss scaling factor
  set f 0.83
  solution Germanium add name=HallEmob const solve val=$f*Emob/100./100. 
  set Ten "Elec*Tensor(1,HallEmob*Bz,-HallEmob*By,-HallEmob*Bz,1,HallEmob*Bx,HallEmob*By,-HallEmob*Bx,1)"
  pdbSet Germanium Qfn Equation [QFElecContinuity Elec=$Ten R=R G=G]
  
  # Hole
  set f 1.8 ;# or 1.2?
  solution Germanium add name=HallHmob const solve val=$f*Hmob/100./100. 
  set Ten "Hole*Tensor(1,HallHmob*Bz,-HallHmob*By,-HallHmob*Bz,1,HallHmob*Bx,HallHmob*By,-HallHmob*Bx,1)"
  pdbSet Germanium Qfp Equation [QFHoleContinuity Hole=$Ten R=R G=G]
  solution Germanium add name=Eval const solve val= [Eval Eea= 4.0 Eg=Eg0]
  solution Germanium add name=Hole const solve val= [ValFermiDirac Ev=Eval Nv=$Nv T=$T]
  
  pdbSet Germanium DevPsi Equation [Poisson er= 16.2]
  pdbSet Germanium DevPsi Abs.Error 1.0e-9
  pdbSet Germanium DevPsi DampValue 0.025
}

}
