namespace eval Germanium {

proc Fischetti args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to the Fischetti reference.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  # Elec Parameters
  pdbSet Germanium XValley Xiu 9.75
  pdbSet Germanium XValley Xid 1.1 ;# using this value from silicon
  pdbSet Germanium LValley Xiu 16.18
  pdbSet Germanium LValley Xid 1.1 ;# using this value from silicon
  # Hole Parameters
  pdbSet Germanium kp_a  [expr 16.18/3.+1.1--3.1] ;#eV using L valley
  pdbSet Germanium kp_b  -2.16 ;#eV
  pdbSet Germanium kp_d  -6.06  ;#eV
  set L -30.53
  set M -4.64
  set N -33.64
  set g1 [expr -($L+2*$M)/2.0]
  set g2 [expr ($L+$g1)/4.0]
  set g3 [expr -$N/6.0]
  pdbSet Germanium kp_g1 $g1
  pdbSet Germanium kp_g2 $g2
  pdbSet Germanium kp_g3 $g3
}

proc SetGermaniumStrainParams args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to my best fit of data.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  pdbSet Germanium Affinity 4.0
  
  pdbSet Germanium kp_delta  0.29 ;#eV
  pdbSet Germanium kp_a0 5.65e-10  ;# m Ge lattice constant
  
  # Compliance Constants
  pdbSet Germanium c11 [expr  9.71e-12 ]
  pdbSet Germanium c12 [expr -2.52e-12 ]
  pdbSet Germanium c44 [expr  24.4e-12 ] ;# Shear moduli for silicon and germanium need further examination
  # P-type Pi Coefficients
  pdbSet Germanium ppi11 [expr 0e-12*10 ]
  pdbSet Germanium ppi12 [expr 0e-12*10 ]
  pdbSet Germanium ppi44 [expr 0e-12*10 ]
  # N-type Pi Coefficients
  pdbSet Germanium npi11 [expr 0e-12*10 ]
  pdbSet Germanium npi12 [expr 0e-12*10 ]
  pdbSet Germanium npi44 [expr 0e-12*10 ]
  # N-type PiezoHall Coefficients
  pdbSet Germanium np11 [expr  0e-12*10 ]
  pdbSet Germanium np12 [expr  0e-12*10 ]
  pdbSet Germanium np44 [expr  0e-12*10 ]
  
  pdbSet Germanium alpha 86.8
  pdbSet Germanium LLongitudinalMass 1.59
  pdbSet Germanium LTransverseMass 0.0815
  
  pdbSet Germanium f 10
  Fischetti
}

}

