namespace eval SiC {
#set low field electron mobility via analytical expression from Farahmand for low field mobility
#paramters for SiC for low field mobility from Farahmand
proc ElecMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=Temp"
  lappend usage "The temperature (T) defaults to an alagator expression of Temp"
  set returns "SiC electron mobility alagator string"
  set description {Computes the temperature dependent electron mobility for SiC}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T ]
  if {$T == ""} { set T Temp }
  set T ($T)

  #parameters for SiC low field mobility copied from AlGaN
  set Smumin 312.1
  set Smumax 1401.3
  set Slowalpha 0.74
  set Sbeta1 -6.51
  set Sbeta2 -2.31
  set Sbeta3 7.07
  set Sbeta4 -0.86
  set SNref 1e17
  
  #build equation for low field mobility
  set Sseg1 "($Smumin)*(exp(log($T/300)*($Sbeta1)))"
  set Sseg2 "($Smumax-$Smumin)*(exp(log($T/300)*($Sbeta2)))"
  set Sseg3 "$SNref*(exp(log($T/300)*($Sbeta3)))"
  set Sseg4 "abs((Doping+1)/$Sseg3)"
  set Sseg5 "$Slowalpha*(exp(log($T/300)*($Sbeta4)))"
  set Sseg6 "exp(log($Sseg4)*($Sseg5))"
  set Sseg7 "1+$Sseg6"
  set Slowfldmob "($Sseg1+(($Sseg2)/($Sseg7)))"
  
  set Shighalpha 6.9502
  set Sn1 7.8138
  set Sn2 0.7897
  set SEcmob 245579.4
  set Svsat 2.02e7
  set SEfield "(sqrt(dot(Qfn,Qfn)+1.0e6))"
  set SEfield_EcRatio "($SEfield)/($SEcmob)"
  set SEn1 "exp(log($SEfield_EcRatio)*($Sn1-1))"
  set SEn2 "exp(log($SEfield_EcRatio)*($Sn2))"
  set Snum "($Slowfldmob+($Svsat*($SEn1)/($SEcmob)))"
  set Sden "(1.0+($Shighalpha*($SEn2))+($SEn1*$SEfield_EcRatio))"
  #return $Slowfldmob
  return 230
}
#set hole mobility as constant
proc HoleMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns "SiC hole mobility alagator string"
  set description {Gives a rough SiC hole mobility}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  return 0.2
}
}

