namespace eval Silicon {
proc KlaassenNAs args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Accept=Accept Donor=Donor Elec=Elec Hole=Hole expand"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Donor concentration (Donor) is assumed to be the alagator field Donor and dopant type As.}
  lappend usage {Accept concentration (Accept) is assumed to be the alagator field Accept.}
  lappend usage {Hole concentration (Hole) is assumed to be the alagator field Hole.}
  lappend usage {Elec concentration (Elec) is assumed to be the alagator field Elec.}
  lappend usage {The expand flag is used to expand the terms into a full string for testing purposes.}
  set returns {alagator string}
  set description {Klaassen, aka Philips, Arsenic Donor Mobility Model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set expand [__Contains $args expand]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  
  #Parameter values for Arsenic
  set KAsEmumax 1417.0
  set KAsEmumin 52.2
  set KAsENref1 9.68e16
  set KAsEalph1 0.68
  set KAsEcD 0.21
  set KAsENrefD 4.0e20
  set KAsEcA 0.50
  set KAsENrefA 7.2e20
  set KAsEtheta_e 2.285
  set KAsEtheta_h 2.247
  
  #Klaussen's fit parameters
  set KAsEme 1.0
  set KAsEmh 1.258
  set KAsEfcw 2.459
  set KAsEfbh 3.828
  
  # Lattice scattering
  set KAsEmuL "($KAsEmumax*(300.0/$T)^$KAsEtheta_e)"
  if { !$expand } {
    term name=KAsEmuL add eqn=$KAsEmuL
  }
  
  # Impurity and Hole Scattering.
  set KAsEmuiN [expr "($KAsEmumax*$KAsEmumax/($KAsEmumax-$KAsEmumin))"]
  set KAsEmuiN "($KAsEmuiN*($T/300.0)^(3.0*$KAsEalph1-1.5))"
  set KAsEmuiC [expr "($KAsEmumax*$KAsEmumin/($KAsEmumax-$KAsEmumin))"]
  set KAsEmuiC "($KAsEmuiC*(300.0/$T)^0.5)"
  set KAsENsc "($Donor+$Accept+$Hole)"
  set KAsEPBH "(1.36e20/$Elec*$KAsEme*($T/300.0)^2.0)"
  set KAsEZ "(1.0+1.0/($KAsEcA+($KAsENrefA/$KAsENsc)^2.0))"
  if { !$expand } {
    term name=KAsEZ add eqn=$KAsEZ
  }
  set KAsEPCW "(3.97e13*((1.0/((KAsEZ)^3.0*$KAsENsc))*($T/300.0)^3.0)^(2.0/3.0))"
  set KAsEP "(1.0/($KAsEfcw/($KAsEPCW)+$KAsEfbh/($KAsEPBH)))"
  if { !$expand } {
  term name=KAsEP add eqn=$KAsEP
  }
  
  set KAsEs1 0.89233
  set KAsEs2 0.41372
  set KAsEs3 0.19778
  set KAsEs4 0.28227
  set KAsEs5 0.005978
  set KAsEs6 1.80618
  set KAsEs7 0.72169
  set KAsE_G "(1.0-$KAsEs1/($KAsEs2+(($T/300.0/$KAsEme)^$KAsEs4)*(KAsEP))^$KAsEs3+$KAsEs5/((($KAsEme*300.0/$T)^$KAsEs7)*KAsEP)^$KAsEs6)"
  if { !$expand } {
  term name=KAsE_G add eqn=$KAsE_G
  }
  
  set KAsEr1 0.7643
  set KAsEr2 2.2999
  set KAsEr3 6.5502
  set KAsEr4 2.3670
  set KAsEr5 -0.01552
  set KAsEr6 0.678
  set KAsE_F "(($KAsEr1*KAsEP^$KAsEr6+$KAsEr2+$KAsEr3*$KAsEme/$KAsEmh)/(KAsEP^$KAsEr6+$KAsEr4+$KAsEr5*$KAsEme/$KAsEmh))"
  if { !$expand } {
  term name=KAsE_F add eqn=$KAsE_F
  }
  
  set KAsENsceff "($Donor+KAsE_G*$Accept+$Hole/KAsE_F)"
  
  set KAsEmuDAh "($KAsEmuiN*$KAsENsc/$KAsENsceff*($KAsENref1/$KAsENsc)^$KAsEalph1+$KAsEmuiC*($Elec+$Hole)/$KAsENsceff)"
  if { !$expand } {
  term name=KAsEmuDAh add eqn=$KAsEmuDAh
  }
  
  # Combine using Mathiesson's rule
  if { $expand } {
    # expand the string by term 10 iterations is just to make sure
    # the whole expression gets expanded. Eval is key to the expand
    set ret "(KAsEmuL*KAsEmuDAh/(KAsEmuL+KAsEmuDAh))"
    for { set i 0 } { $i < 10 } { incr i } {
      eval set ret [string map {K "$\K"} $ret]
    }
    return $ret
  } else {
    return "(KAsEmuL*KAsEmuDAh/(KAsEmuL+KAsEmuDAh))"
  }
}


}
