namespace eval Silicon {
proc KlaassenNP args {
	set helpString "\u001b\[1m\u001b\[33mKlaassen Phosphorus Donor Mobility Model:\u001b\[0m\n\
		  Usage: \[ KlaassenNP Temperature Acceptors Donors Hole Elec \]\n\
		  The temperature parameter is either a fixed value or an Alagator solution variable \n\
		  The Donors, Acceptors, Hole, Elec parameters are either a fixed value or an Alagator solution variable \n\
		  Sets: Sets a solution variable EMob to be used in current equation \n\
      Returns: Nothing"
  foreach {opt} $args {
    if {$opt == "-help"} {
      puts $helpString
      return ""
    }
  }
  if {[llength $args] != 5} {
    return -code error "\u001b\[1m\u001b\[31mInvalid Number of Arguments\u001b\[0m \n$helpString"
  }
  set T         [lindex $args 0]
  set Acceptors [lindex $args 1]
  set Donors    [lindex $args 2]
  set Hole      [lindex $args 3]
  set Elec      [lindex $args 4]
  
  #Parameter values for Phospherous
  set KEmumax 1414.0
  set KEmumin 68.5
  set KENref1 9.2e16
  set KEalph1 0.711
  set KEcD 0.21
  set KENrefD 4.0e20
  set KEcA 0.50
  set KENrefA 7.2e20
  set KEtheta_e 2.285
  set KEtheta_h 2.247
  
  #Klaussen's fit parameters
  set KEme 0.2
  set KEmh 1.258
  set KEfcw 2.459
  set KEfbh 3.828
  
  # Lattice scattering
  set KEmuL [expr "($KEmumax*(300.0/$T)**$KEtheta_e)"]
  
  # Impurity and Hole Scattering.
  set KEmuiN [expr "($KEmumax*$KEmumax/($KEmumax-$KEmumin))"]
  set KEmuiN "($KEmuiN*($T/300.0)**(3.0*$KEalph1-1.5))"
  set KEmuiC [expr "($KEmumax*$KEmumin/($KEmumax-$KEmumin))"]
  set KEmuiC "($KEmuiC*(300.0/$T)**0.5)"
  set KENsc "($Donors+$Acceptors+$Hole)"
  set KEPBH "(1.36e20/$Elec*$KEme*($T/300.0)**2.0)"
  set KEfoo "(1.0+1.0/($KEcA+($KENrefA/$KENsc)**2.0))"
  set KEZ [expr $KEfoo]
  set KEPCW "(3.97e13*((1.0/(($KEZ)**3.0*$KENsc))*($T/300.0)**3.0)**(2.0/3.0))"
  set KEfoo [expr "(1.0/($KEfcw/($KEPCW)+$KEfbh/($KEPBH)))"]
  set KEP $KEfoo
  
  set KEs1 0.89233
  set KEs2 0.41372
  set KEs3 0.19778
  set KEs4 0.28227
  set KEs5 0.005978
  set KEs6 1.80618
  set KEs7 0.72169
  set KEfoo "(1.0-$KEs1/($KEs2+(($T/300.0/$KEme)**$KEs4)*($KEP))**$KEs3+$KEs5/((($KEme*300.0/$T)**$KEs7)*$KEP)**$KEs6)"
  set KE_G $KEfoo
  
  set KEr1 0.7643
  set KEr2 2.2999
  set KEr3 6.5502
  set KEr4 2.3670
  set KEr5 -0.01552
  set KEr6 0.678
  set KEfoo "(($KEr1*$KEP**$KEr6+$KEr2+$KEr3*$KEme/$KEmh)/($KEP**$KEr6+$KEr4+$KEr5*$KEme/$KEmh))"
  set KE_F $KEfoo
  
  set KENsceff "($Donors+$KE_G*$Acceptors+$Hole/$KE_F)"
  
  set KEfoo "($KEmuiN*$KENsc/$KENsceff*($KENref1/$KENsc)**$KEalph1+$KEmuiC*($Elec+$Hole)/$KENsceff)"
  set KEmuDAh [expr $KEfoo]
  
  # Combine using Mathiesson's rule
  set KEfoo "($KEmuL*$KEmuDAh/($KEmuL+$KEmuDAh))"
  return "[expr $KEfoo] $KEmuL $KEmuDAh"
}

set T 300; set NA 1e10; set ND 1e16; set Hole 1e20; set Elec 1e20
puts [KlaassenNP $T $NA $ND $Hole $Elec]

set T 300; set NA 1e16; set ND 1e18; set Hole 1e14; set Elec 1e16
puts [KlaassenNP $T $NA $ND $Hole $Elec]

window
foreach Hole "1e13 1e14 1e15 1e16 1e17 1e18" {
  set T 300; set NA 1e10; set ND 1e18; set Elec 1e18
  set tmp [KlaassenNP $T $NA $ND $Hole $Elec]
  puts $tmp
  #chart graph= 1 curve= 1  xval= log10($Hole) yval= [lindex $tmp 0]
  #chart graph= 1 curve= 2  xval= log10($Hole) yval= [expr 1.0 / ( 1.0/([lindex $tmp 1]*2) + 1.0/[lindex $tmp 2]) ]
  chart graph= 1 curve= 2  xval= log10($Hole) yval= [expr ( [lindex $tmp 0] / (1.0 / ( 1.0/([lindex $tmp 1]/1.1) + 1.0/[lindex $tmp 2]) ) - 1)*100 ]
}


}
