namespace eval Silicon {
proc KlaassenPB args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Accept=Accept Donor=Donor Elec=Elec Hole=Hole expand"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Accept concentration (Accept) is assumed to be the alagator field Accept and dopant type boron.}
  lappend usage {Donor concentration (Donor) is assumed to be the alagator field Donor.}
  lappend usage {Hole concentration (Hole) is assumed to be the alagator field Hole.}
  lappend usage {Elec concentration (Elec) is assumed to be the alagator field Elec.}
  lappend usage {The expand flag is used to expand the terms into a full string for testing purposes.}
  set returns {alagator string}
  set description {Klaassen, aka Philips, Phosphorus Donor Mobility Model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set expand [__Contains $args expand]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  
  #Parameter values for Boron
  set KHmumax 470.5
  set KHmumin 44.9
  set KHNref1 2.23e17
  set KHalph1 0.719
  set KHcD 0.21
  set KHNrefD 4.0e20
  set KHcA 0.50
  set KHNrefA 7.2e20
  set KHtheta_e 2.285
  set KHtheta_h 2.247
  
  #Klaussen's fit parameters
  set KHme 1.0
  set KHmh 1.258
  set KHfcw 2.459
  set KHfbh 3.828
  
  # Lattice scattering
  set KHmuL "($KHmumax*(300.0/$T)^$KHtheta_h)"
  if { !$expand } {
    term name=KHmuL add eqn=$KHmuL
  }
  
  # Impurity and Elec Scattering.
  set KHmuiN [expr "($KHmumax*$KHmumax/($KHmumax-$KHmumin))"]
  set KHmuiN "($KHmuiN*($T/300.0)^(3.0*$KHalph1-1.5))"
  set KHmuiC [expr "($KHmumax*$KHmumin/($KHmumax-$KHmumin))"]
  set KHmuiC "($KHmuiC*(300.0/$T)^0.5)"
  set KHNsc "($Donor+$Accept+$Elec)"
  set KHPBH "(1.36e20/$Hole*$KHmh*($T/300.0)^2.0)"
  set KHZ   "(1.0+1.0/($KHcD+($KHNrefA/$KHNsc)^2.0))"
  if { !$expand } {
    term name=KHZ add eqn=$KHZ
  }
  set KHPCW "(3.97e13*((1.0/(KHZ^3.0*$KHNsc))*($T/300.0)^3.0)^(2.0/3.0))"
  set KHP   "(1.0/($KHfcw/($KHPCW)+$KHfbh/($KHPBH)))"
  if { !$expand } {
    term name=KHP add eqn=$KHP
  }
  
  set KHs1 0.89233
  set KHs2 0.41372
  set KHs3 0.19778
  set KHs4 0.28227
  set KHs5 0.005978
  set KHs6 1.80618
  set KHs7 0.72169
  set KH_G  "(1.0-$KHs1/($KHs2+(($T/300.0/$KHmh)^$KHs4)*(KHP))^$KHs3+$KHs5/((($KHmh*300.0/$T)^$KHs7)*KHP)^$KHs6)"
  if { !$expand } {
  term name=KH_G add eqn=$KH_G
  }
  
  set KHr1 0.7643
  set KHr2 2.2999
  set KHr3 6.5502
  set KHr4 2.3670
  set KHr5 -0.01552
  set KHr6 0.678
  set KH_F  "(($KHr1*KHP^$KHr6+$KHr2+$KHr3*$KHmh/$KHme)/(KHP^$KHr6+$KHr4+$KHr5*$KHmh/$KHme))"
  if { !$expand } {
  term name=KH_F add eqn=$KH_F
  }
  
  set KHNsceff "($Accept+KH_G*$Donor+$Elec/KH_F)"
  
  set KHmuDAh "($KHmuiN*$KHNsc/$KHNsceff*($KHNref1/$KHNsc)^$KHalph1+$KHmuiC*($Elec+$Hole)/$KHNsceff)"
  if { !$expand } {
  term name=KHmuDAh add eqn=$KHmuDAh
  }
  
  # Combine using Mathiesson's rule
  if { $expand } {
    # expand the string by term 10 iterations is just to make sure
    # the whole expression gets expanded. Eval is key to the expand
    set ret "(KHmuL*KHmuDAh/(KHmuL+KHmuDAh))"
    for { set i 0 } { $i < 10 } { incr i } {
      eval set ret [string map {K "$\K"} $ret]
    }
    return $ret
  } else {
    return "(KHmuL*KHmuDAh/(KHmuL+KHmuDAh))"
  }
}


}
