
namespace eval Silicon {
proc __2DElecDP { Elecs } {
  return [getQfnTensor [__getXmobs] $Elecs "" 0 0 0 "" Silicon]
}


proc Get3DElecDPTensor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Elecs=? Mobs=? miller=(001)<100>"
  lappend usage "Vector of electron populations of length three called Elecs"
  lappend usage "Tensor of mobilites with dimension 3x3. Where the row is the valley direction and the column is the current direction."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set returns "Tensor list containing the relative conductivity changes for holes"
  set description {Computes the normalized conductivity change for holes according to the linear piezoresistance model}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set Elecs [__GetValue $args Elecs "$name Error: No electron population vector (Elecs) provided"]
  set Mobs [__GetValue $args Mobs "$name Error: No mobility tensor (Mobs) provided"]
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }

  set TensorList ""
  # this is if we figure out we need a tensor elec
  # for { set i 0 } { $i < 3 } { incr i } {
  #   set TenListTmp ""
  #   # lappend TenListTmp [lindex [GetMobilityTensor [lindex [__getXmobs "0 0 0 0 0 0" $miller $Elecs Silicon ] $i ] ] 0]
  #   # lappend TenListTmp [lindex [GetMobilityTensor [lindex $Mobs  $i ] ] 0]
  #   # lappend TenListTmp [lindex [GetMobilityTensor [lindex $Elecs $i ] ] 0]
  #   lappend TenListTmp [lindex [GetMobilityTensor [lindex $Mobs $i ] ] 0]
  #   lappend TenListTmp [lindex [__RotateTensor [GetMobilityTensor [lindex $Elecs $i ] ] $miller] 0]
  #   set TenTmp [3DTensorDyadicMult $TenListTmp] 
  #   lappend TensorList "[lindex $TenTmp 0]"
  # }
  # Sticking to the simplier method for now
  # set Elecs [__RotateVector3 $Elecs $miller]
  for { set i 0 } { $i < 3 } { incr i } {
    set TenListTmp ""
    # lappend TenListTmp [lindex [GetMobilityTensor [lindex [__getXmobs "0 0 0 0 0 0" $miller $Elecs Silicon ] $i ] ] 0]
    lappend TenListTmp [lindex [GetMobilityTensor [lindex $Mobs $i ] ] 0]
    # only rotate carrier tensor, mobility tensor includes splitting
    if { [llength [lindex $Elecs $i ]] == 1 } {
      lappend TenListTmp [lindex [GetCarrierTensor [lindex $Elecs $i] ] 0]
    } else {
      lappend TenListTmp [lindex [GetMobilityTensor [lindex $Elecs $i ] ] 0]
    }
    set TenTmp [3DTensorDyadicMult $TenListTmp]

    lappend TensorList "[lindex $TenTmp 0]"
  }

  return [__RotateTensorDiag [3DTensorAdd $TensorList] $miller]

}

proc Get3DElecDPTensorPiezoHall args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Elecs=? Mobs=? RawMobs=? HallMob=? B=? miller=(001)<100>"
  lappend usage "Vector of electron populations of length three called Elecs"
  lappend usage "Tensor of mobilites with dimension 3x3. Where the row is the valley direction and the column is the current direction."
  lappend usage "The value of the Hall mobility (HallMob) is required."
  lappend usage "A magnetic field vecotr (B) is required and length three for {Bx By Bz}."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set returns "Tensor list containing the relative conductivity changes for holes"
  set description {Computes the normalized conductivity change for holes according to the linear piezoresistance model}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set Elecs [__GetValue $args Elecs "$name Error: No electron population vector (Elecs) provided"]
  set RawMobs [__GetValue $args RawMobs "$name Error: No mobility tensor (RawMobs) provided"]
  set Mobs [__GetValue $args Mobs "$name Error: No mobility tensor (Mobs) provided"]
  set HallMob [__GetValue $args HallMob "$name Error: No Hall mobility (HallMob) provided"]
  set B [__GetValue $args B "$name Error: No magnetic field vector (B) provided"]
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }

  set TensorList [Get3DElecDPTensor Elecs=$Elecs Mobs=$Mobs miller=$miller]
  set HallTensorList ""
  for { set i 0 } { $i < 3 } { incr i } {
    lappend HallTensorList [lindex [GetHallTensor [lindex $Mobs $i ] [lindex $RawMobs $i ] [lindex $Elecs $i ] $HallMob $B ] 0]
  }
  lappend TensorList [lindex [__RotateTensor [3DTensorAdd $HallTensorList] $miller] 0]
  return [3DTensorAdd $TensorList]
}

proc Get3DElecDPTensorDensityGradient { Elecs StressVector6 StressRotation CrystalRotation } {
  set TensorList ""
  for { set i 0 } { $i < 3 } { incr i } {
    set TenListTmp ""
    # Both ways seem valid for rotation of mobility
    lappend TenListTmp [lindex [GetMobilityTensor [lindex [__getXmobs $StressVector6 $StressRotation $CrystalRotation $Elecs Silicon ] $i ] ] 0]
    lappend TenListTmp [lindex [GetAnisoCarrierTensor [lindex $Elecs $i ] ] 0]
    set TenTmp  [3DTensorDyadicMult $TenListTmp]
    lappend TensorList "[lindex $TenTmp 0]"
  }
  return [3DTensorAdd $TensorList]
}

# proc Get2DElecDPTensor { Elecs StressVector6 CrystalRotation StressRotation } {
#   return [3DTensorTo2D [3DElecDP $Elecs $StressVector6 $CrystalRotation $StressRotation ]]
# }


}

