namespace eval Silicon {

proc VanDeWalle args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to the Van Der Walle reference.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  pdbSet Silicon kp_delta  0.044  ;#eV
  pdbSet Silicon kp_a0 5.43e-10 ;# m Si lattice constant
  # Elec Parameters
  pdbSet Silicon XValley Xiu 9.2
  pdbSet Silicon XValley Xid 1.13
  # Hole Parameters
  pdbSet Silicon kp_a  2.46 ;#eV
  pdbSet Silicon kp_b  -2.35 ;#eV
  pdbSet Silicon kp_d  -5.32  ;#eV
}

proc Fischetti args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to the Fischetti reference.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  pdbSet Silicon kp_delta  0.044 ;#eV
  pdbSet Silicon kp_a0 5.43e-10  ;# m Si lattice constant
  # Elec Parameters
  pdbSet Silicon XValley Xiu 10.5
  pdbSet Silicon XValley Xid 1.1
  # Hole Parameters
  pdbSet Silicon kp_a  2.1 ;#eV
  pdbSet Silicon kp_b  -2.33 ;#eV
  pdbSet Silicon kp_d  -4.75  ;#eV
  set L -5.53
  set M -3.64
  set N -8.32
  set g1 [expr -($L+2*$M)/2.0]
  set g2 [expr ($L+$g1)/4.0]
  set g3 [expr -$N/6.0]
  pdbSet Silicon kp_g1 $g1
  pdbSet Silicon kp_g2 $g2
  pdbSet Silicon kp_g3 $g3
}

proc Fridel args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to the Fridel reference.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  pdbSet Silicon kp_delta  0.044  ;#eV
  pdbSet Silicon kp_a0 5.43e-10 ;# m Si lattice constant
  # Elec Parameters
  pdbSet Silicon XValley Xiu 8.47
  pdbSet Silicon XValley Xid 1.03
  # Hole Parameters
  pdbSet Silicon kp_g1 4.285
  pdbSet Silicon kp_g2 0.339
  pdbSet Silicon kp_g3 1.46
  pdbSet Silicon kp_a  2.06 ;#eV
  pdbSet Silicon kp_b  -2.27 ;#eV
  pdbSet Silicon kp_d  -3.69  ;#eV
}

proc SetSiliconStrainParams args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to my best fit of data.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  pdbSet Silicon Affinity 4.05
  set L -5.53
  set M -3.64
  set N -8.32
  set g1 [expr -($L+2*$M)/2.0]
  set g2 [expr ($L+$g1)/4.0]
  set g3 [expr -$N/6.0]
  pdbSetDouble Silicon kp_g1 $g1
  pdbSetDouble Silicon kp_g2 $g2
  pdbSetDouble Silicon kp_g3 $g3
  
  pdbSetDouble Silicon kp_a0 5.43e-10 ;# m Si lattice constant
  # pdbSetDouble Silicon kp_g1 4.22
  # pdbSetDouble Silicon kp_g2 0.39
  # pdbSetDouble Silicon kp_g3 1.44
  pdbSetDouble Silicon kp_a  2.46 ;#eV
  pdbSetDouble Silicon kp_b  -2.35 ;#eV
  pdbSetDouble Silicon kp_d  -5.3  ;#eV
  pdbSetDouble Silicon kp_delta  0.044  ;#eV
  
  pdbSetDouble Silicon kp_a0 5.43e-10 ;# m Si lattice constant
  pdbSetDouble Silicon kp_g1 4.285
  pdbSetDouble Silicon kp_g2 0.339
  pdbSetDouble Silicon kp_g3 1.46
  pdbSetDouble Silicon kp_b  -1.36 ;#eV
  pdbSetDouble Silicon kp_d  -3.09  ;#eV
  Fridel
  pdbSet Silicon XValley Xiup 5.2
  # Compliance Constants
  pdbSet Silicon c11 [expr  7.68e-12 ]
  pdbSet Silicon c12 [expr -2.14e-12 ]
  # pdbSet Silicon c44 [expr  12.6e-12 ] ;# not sure which shear modulus value is right
  pdbSet Silicon c44 [expr  19.2e-12 ]
  # P-type Pi Coefficients
  pdbSet Silicon ppi11 [expr   6.6e-12*10 ]
  pdbSet Silicon ppi12 [expr  -1.1e-12*10 ]
  pdbSet Silicon ppi44 [expr 138.1e-12*10 ]
  # N-type Pi Coefficients
  pdbSet Silicon npi11 [expr -102.2e-12*10 ]
  pdbSet Silicon npi12 [expr   53.4e-12*10 ]
  pdbSet Silicon npi44 [expr  -13.6e-12*10 ]
  # N-type PiezoHall Coefficients
  pdbSet Silicon np11 [expr  -88.0e-12*10 ]
  pdbSet Silicon np12 [expr   45.0e-12*10 ]
  pdbSet Silicon np44 [expr  -11.0e-12*10 ]
  
  pdbSet Silicon alpha 53.0
  pdbSet Silicon XLongitudinalMass 0.9161
  pdbSet Silicon XTransverseMass 0.1905
  pdbSet Silicon XTotalMass 0.26
  
  pdbSet Silicon f 10
  #VanDeWalle
  #Fischetti
}

proc SetSiliconStrainParams100 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to my best fit of data for 100.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  SetSiliconStrainParams
  pdbSet Silicon kp_a  2.057 ;#eV
  pdbSet Silicon kp_b  -1.8956 ;#eV
  pdbSet Silicon kp_d  -4.75  ;#eV
  pdbSet Silicon kp_g1 6.48
  pdbSet Silicon kp_g2 0.2697
  pdbSet Silicon kp_g3 2.107
}

proc SetSiliconStrainParams110 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns {nothing}
  set description {Sets deformation parameters according to my best fit of data for 110.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  SetSiliconStrainParams
  pdbSet Silicon kp_a  3.847
  pdbSet Silicon kp_b  -2.028
  pdbSet Silicon kp_d  -4.75
  pdbSet Silicon kp_g1 6.242
  pdbSet Silicon kp_g2 0.382
  pdbSet Silicon kp_g3 1.342
}

proc SetStrainParamsList args {
  set name [lindex [info level 0] 0]
  lappend usage "$name list"
  set returns {nothing}
  set description {Sets deformation parameters according to list.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set tmp [lindex $args 0]
  if { [lindex $tmp 0] != "" } {
    pdbSet Silicon kp_a  [expr [lindex $tmp 0]] ;#eV 
  }
  if { [lindex $tmp 1] != "" } {
    pdbSet Silicon kp_b  [expr [lindex $tmp 1]] ;#eV
  }
  if { [lindex $tmp 2] != "" } {
    pdbSet Silicon kp_d  [expr [lindex $tmp 2]]  ;#eV
  }
  if { [lindex $tmp 3] != "" } {
    pdbSet Silicon kp_g1  [expr [lindex $tmp 3]] 
  }
  if { [lindex $tmp 4] != "" } {
    pdbSet Silicon kp_g2  [expr [lindex $tmp 4]] 
  }
  if { [lindex $tmp 5] != "" } {
    pdbSet Silicon kp_g3  [expr [lindex $tmp 5]] 
  }
  if { [lindex $tmp 6] != "" } {
    pdbSet Silicon XValley Xiu   [expr [lindex $tmp 6]] 
  }
  if { [lindex $tmp 7] != "" } {
    pdbSet Silicon XValley Xid   [expr [lindex $tmp 7]] 
  }
  if { [lindex $tmp 8] != "" } {
    pdbSet Silicon f  [expr [lindex $tmp 8]] 
  }
  if { [lindex $tmp 9] != "" } {
    pdbSet Silicon alpha [expr [lindex $tmp 9]]
  }
}

}

