#all necessary procedures listed
#prework/ramping completed
#write procedure for calculating the dependent variable value (yCalc)

proc PredCor {in1 max1 inc err conName supName gName cName yCalc} {

  #in1 - first x value
  #max1 - final x value
  #inc - first step
  #err - max allowed error (percent)
  #conName - contact name=___
  #supName - ___ supply=
  #gName - chart graph=___
  #cName - curve=___
  #yCalc - procedure to calculate y 

  set max [expr {$max1 + .00001}]

  set loop 0

  set err [expr {$err / 100.0}]

  contact name=$conName $supName supply=$in1
  device
  device store

  set dep [$yCalc]
  chart graph=$gName curve=$cName xval=$in1 yval=$dep

  #set first point values as x1 and y1
  set x1 $in1
  set y1 $dep

  #set first reference value to independent variable val
  #set next indep var to current val + initial increase
  set ref $in1
  set in [expr {$in1 + $inc}]

  #loop until max
  while {$loop == 0} {

    set error 1
    device restore
    #in case of matrix issue
    while {$error == 1} {

      set error 0

      if {[catch {contact name=$conName $supName supply=$in
        device}]} {
        
        device restore
        set error 1

        #cut increase in half and add it to previous indep var val
        set inc [expr {$inc/2}]
        set in [expr {$ref + $inc}]
      }
    }
    device store

    #calc dep value after above in value
    set dep [$yCalc]

    #store results for later graphing
    set ref $in
    set ina $in
    set depa $dep

    #set second point values as x2 and y2
    set x2 $in
    set y2 $dep
    
    #get slope
    set slope [expr {($y2 - $y1)/($x2 - $x1)}]
    #get b
    set b [expr {$y2-($slope*$x2)}]

    #get next indepvar value 
    set in [expr {$x2 + $inc}] 
    if {$in > $max} {
      set in $max
    }
    #set projected y on line using eq just constructed
    set p1 [expr {($slope*$in) + $b}]

    #get actual next y value
    set error 1
    device restore
    while {$error == 1} {

      set error 0

      if {[catch {contact name=$conName $supName supply=$in
        device} ] } {
        
        device restore
        set error 1

        #cut increase in half and add it to previous indep var val
        set inc [expr {$inc/2}]
        set in [expr {$ref + $inc}]

        #set projected y on line using eq just constructed
        #use updated indep variable value
        set p1 [expr {($slope*$in) + $b}]
      }
    }
    #calc dep value after in from above; don't graph
    set dep [$yCalc]

    #store results for later graphing
    set inb $in
    set depb $dep

    #difference between actual point and projected point
    set error [expr {$p1 - $dep}]

    set maxerr [expr {$err*$dep}]

    if {$error < 0} {
      set abserror [expr {$error*(-1)}]
    }
    if {$error > 0} {
      set abserror $error
    }
    if {$error == 0} {
      set abserror $error
    }
    if {$abserror < $maxerr} {
      set go 1
    }
    if {$abserror >= $maxerr} {
      set go 0
    }

    #error within allowance; can proceed
    if {$go == 1} {
      device store

      #calculate ideal next step size
      set diffsq [expr {$inc*$inc}]
      set secder [expr {(2*$error)/$diffsq}]
      if {$secder < 0} {
        set secder [expr {$secder*(-1)}] 
      }
      set inc [expr {sqrt((2*$maxerr)/$secder)}]
      set in1 $in
      set x1 $in1
      set y1 $dep

      #graph ina with depa
      chart graph=$gName curve=$cName xval=$ina yval=$depa
      #graph inb with depb
      chart graph=$gName curve=$cName xval=$inb yval=$depb
              
    }

    #error too high; cannot proceed
    if {$go == 0} {
      device restore
      #graph ina with depa
      chart graph=$gName curve=$cName xval=$ina yval=$depa

      #step back and try again
      set inc [expr {$inc*.5}]
      set x1 $x2
      set y1 $y2
    }
    
    #finished
    if {$in1 >= $max1} {
      set loop 1
      puts "max reached"
    }

    if {$in1 < $max1} {
      set in [expr {$in1 + $inc}] 
      set ref $in1

      #take it to the end
      if {$in >= $max} {
        set loop 1
        set in $max

        set error 1

        #calc until converges and max has been reached
        device restore
        while {($error == 1) || ($in < $max)} {
          set error 0
          if {[catch {contact name=$conName $supName supply=$in
          device} ] } {  
            device restore     
            set error 1
            set inc [expr {$inc/2}]
            set in [expr {$ref + $inc}]
          }
          set dep [$yCalc]

          #graph each success until the end
          if {$error == 0} {
            chart graph=$gName curve=$cName xval=$in yval=$dep
            set in [expr {$in + $inc}]
            device store
          }
        } 
        puts "max reached"
      }        
    }
  }
}
