
proc initial_solve args {
  set name [lindex [info level 0] 0]
  lappend usage "$name solve= {Qfn Qfp} potential=DevPsi elec=Elec hole=Hole Doping=Doping"
  lappend usage "Unless otherwise specified the solve is performed on the DevPsi solution variable"
  lappend usage "The solution variables Qfn and Qfp are solved unless otherwise specified with the solve list, the potential is solved no matter what"
  lappend usage "Elec is used for the electron concentration (elec) unless otherwise specified"
  lappend usage "Hole is used for the hole concentration (hole) unless otherwise specified"
  lappend usage "Doping is used for the doping concentration (doping) unless otherwise specified"
  set returns "nothing"
  set description "Performs an initial newton solve on all materials for DevPsi and then runs a device_cache command"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set potential [__GetValue $args potential]
  if {$potential == ""} { set potential DevPsi }
  set elec [__GetValue $args elec]
  if {$elec == ""} { set elec Elec }
  set hole [__GetValue $args hole]
  if {$hole == ""} { set hole Hole }
  set Doping [__GetValue $args Doping]
  if {$Doping == ""} { set Doping Doping }
  set solve [__GetValue $args solve]
  if {$solve == ""} { set solve {Qfn Qfp} }

  # set all contact supply voltages to 0
  foreach c [contact list] {
    if { [string first "Reflect" $c] == -1 } {
      contact supply= 0.0 name= $c
    }
  }

  sel z= 0 name=Qfn
  sel z= 0 name=Qfp

  set fil structures/cache/[__GetStateHash].str
  if { ![file exists $fil] } {
    foreach mat [mater list bulk] {
      if { $mat == "Metal" || $mat == "Nitride" || $mat == "Oxide" } { continue; }
      newton $mat eqn=($Doping)-($elec)+($hole) var=$potential damp=0.025
    }
  }
  pdbSet Math iterLimit 200
  # Check solutions, disable generation recombination if Qfn and Qfp are not present
  if { [string first Qfn $solve] == -1 || [string first Qfp $solve] == -1 } {
    puts "WARNING: Solutions for both Qfn and Qfp are needed for generation and recombination"
    foreach m [mater list] {
      solution $m name= R const nosolve val= 0
      solution $m name= G const nosolve val= 0
    }
  }
  # Solve only DevPsi
  foreach s $solve {
    if { $s == $potential } { continue }
    solution name= $s nosolve
  }
  if { [catch {device init} ] } {
    puts "Failed initial condition"
    exit 
  }
  # Solve everything else
  foreach s $solve {
    solution name= $s solve
  }
  if { [device_cache init n=2 norunonexists] } { 
    puts "Failed initial condition"
    exit 
  }
  pdbSet Math iterLimit 30
  device init

  return ""
}

proc initial_solve_sg args {
  set name [lindex [info level 0] 0]
  lappend usage "$name solve= {Elec Hole} potential=DevPsi Doping=Doping"
  lappend usage "Unless otherwise specified the solve is performed on the DevPsi solution variable"
  lappend usage "The solution variables Elec and Hole are solved unless otherwise specified with the solve list, the potential is solved no matter what"
  lappend usage "Doping is used for the doping concentration (Doping) unless otherwise specified"
  set returns "nothing"
  set description "Performs an initial newton solve on all materials for DevPsi and then runs a device_cache command"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set potential [__GetValue $args potential]
  if {$potential == ""} { set potential DevPsi }
  set Doping [__GetValue $args Doping]
  if {$Doping == ""} { set Doping Doping }
  set solve [__GetValue $args solve]
  if {$solve == ""} { set solve {Elec Hole} }

  # set all contact supply voltages to 0
  foreach c [contact list] {
    if { [string first "Reflect" $c] == -1 } {
      contact supply= 0.0 name= $c
    }
  }

  sel z= ($Doping>0)?($Doping):1e5 name=Elec
  sel z= ($Doping<0)?($Doping):1e5 name=Hole

  set fil structures/cache/[__GetStateHash].str
  if { ![file exists $fil] } {
    foreach mat [mater list bulk] {
      if { $mat == "Metal" || $mat == "Nitride" || $mat == "Oxide" } { continue; }
      newton $mat eqn=($Doping)-(ni*exp(-DevPsi/0.025))+(ni*exp(DevPsi/0.025)) var=$potential damp=0.025
    }
  }
  sel z= (ni*exp(DevPsi/0.025))  name= Hole
  sel z= (ni*exp(-DevPsi/0.025)) name= Elec
  pdbSet Math iterLimit 200
  # Check solutions, disable generation recombination if Elec and Hole are not present
  if { [string first Elec $solve] == -1 || [string first Hole $solve] == -1 } {
    puts "WARNING: Solutions for both Elec and Hole are needed for generation and recombination"
    foreach m [mater list] {
      solution $m name= R const nosolve val= 0
      solution $m name= G const nosolve val= 0
    }
  }
  # Solve everything else
  foreach s $solve {
    solution name= $s !negative solve
  }
  if { [device_cache init n=2 norunonexists] } { 
    puts "Failed initial condition"
    exit 
  }
  pdbSet Math iterLimit 30
  device init

  return ""
}

