#all necessary procedures listed
#prework/ramping completed
#write procedure for calculating the dependent variable value (yCalc)

proc smooth_ramp args {
  set name [lindex [info level 0] 0]
  lappend usage "$name start end maxinc conName yCalc plotAndStore err"
  set returns "nothing"
  set description "start - first x value, end - final x value, maxinc - first step, conName - contact name=___, yCalc - procedure to calculate y, plotAndStore - procedure to plot with passed x value and return associated y values, err - max allowed error (percent)"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  lassign $args start end maxinc conName yCalc plotAndStore err
  if { $err == "" } { set err 10; }
  
  
  set err [expr {$err / 100.0}]
  set xvals ""
  set yvals ""
  
  contact name=$conName supply=$start
  device_cache
  device store
  
  lappend xvals $start
  lappend yvals [$plotAndStore $start]
  
  #set first point values as x1 and y1
  set x1 $start
  set y1 [$yCalc]
  
  #set first reference value to independent variable val
  #set next indep var to current val + initial increase
  set current $start
  set inc $maxinc
  set attempt [expr {$start + $inc}]
  
  while {$attempt < $end} {
    
    device restore
    #in case of matrix issue
    contact name=$conName supply=$attempt
    while { [device_cache] } {
      device restore
      #cut increase in half and add it to previous indep var val
      set inc [expr $inc/2.]
      set attempt [expr {$current + $inc}]
      contact name=$conName supply=$attempt
    }
    device store
    
    # store values and plot
    set current $attempt
    lappend xvals $current
    lappend yvals [$plotAndStore $current]
    
    #set second point values as x2 and y2
    set x2 $current
    set y2 [$yCalc]
    
    #get slope
    set slope [expr {($y2 - $y1)/($x2 - $x1)}]
    set b [expr {$y2-($slope*$x2)}]
    
    # get next indepvar value
    set attempt [expr ($x2 + $inc) > $end ? $end : ($x2 + $inc)]
    # set projected y on line using eq just constructed
    set p1 [expr {($slope*$attempt) + $b}]
    
    #get actual next y value
    device restore
    contact name=$conName supply=$attempt
    while { [device_cache] } {
      device restore
      #cut increase in half and add it to previous indep var val
      set inc [expr {$inc/2.}]
      set attempt [expr {$current + $inc}]
      contact name=$conName supply=$attempt
      
      #set projected y on line using eq just constructed
      #use updated indep variable value
      set p1 [expr {($slope*$attempt) + $b}]
    }
    #calc dep value after in from above; don't graph
    set dep [$yCalc]
    
    # difference between actual point and projected point
    set error [expr {$p1 - $dep}]
    set maxerr [expr {$err*$dep}]
    
    #error within allowance; can proceed
    if { abs($error) < $maxerr } {
      device store
      
      #calculate ideal next step size
      set diffsq [expr {$inc*$inc}]
      set secder [expr abs((2*$error)/$diffsq)]
      set inc [expr {sqrt((2*$maxerr)/$secder)}]
      #set inc [expr $inc > $maxinc ? $maxinc : $inc]
      set x1 $attempt
      set y1 $dep
      lappend xvals $attempt
      lappend yvals [$plotAndStore $attempt]
    } else {
      set inc [expr $inc/2.]
      set x1 $x2
      set y1 $y2
    }
    
    if {[expr $attempt + $inc] >= $end} {
      # linear ramp the final steps
      lassign [linear_ramp $conName $attempt $end $inc $plotAndStore ] xtmp ytmp
      set xvals [concat $xvals $xtmp]
      set yvals [concat $yvals $ytmp]
      break
    }
    set attempt [expr $attempt + $inc]
  }
  puts "end reached"
  return "{$xvals} {$yvals}"
}








