#### LARS test procs ####
#expr ([lindex y_del $i] - $y0)/($delta)
# These are to optimization procs
proc WT {num file_name} {
  set fo [open $file_name a]
  puts $fo "$num starts waiting"
  flush $fo
  after ${num}000
  puts $fo "waited for $num sec"
  close $fo
}

proc wait_srun {job_ID srun_name output} {
  set of [open $output a]
  puts $of "# looking at job $job_ID"
  after 100    ;# wait to make sure that the srun steps finish submitting and will show up when we look for them
  
  set no_run 0
  
  while { $no_run < 5 } {
    if { ![catch {exec sh -c "sacct --jobs=$job_ID --format=jobname,jobid,uid,state | grep $srun_name | grep -cE 'RUNNING|PENDING' "} r_val ] } {
      puts $of "# there are still $r_val processes running"
      flush $of
      set no_run 0
    } else {
      puts $of "# nothing running $no_run"
      flush $of
      incr no_run
    }
    after 200
  }
  puts $of "# all sruns are done!"
  flush $of
}

proc run_errorfxn_y0 { errorfxn errorPath params o_file y_type } {
  set of [open $o_file a]
  puts $of "# we are inside run_err_y0, parmas: $params"
  puts $of "# the errorfxn is: $errorfxn"
  puts $of "# sourcing the error function from $errorPath"
  flush $of
  source $errorPath
  if { ! [ catch [ set y [ $errorfxn $params ] ] ] } {
    puts $of "error in $y_type"
    flush $of
    exit
  } else {
    puts $of "set y0 $y"
    flush $of
  }
}

proc run_errorfxn_y1 { errorfxn errorPath params o_file var_num } {
  set of [open $o_file a]
  puts $of "# we are inside run_err_y1 $var_num, params: $params"
  
  # sourcing the error function
  puts "sourcing the error function from $errorPath"
  source $errorPath
  
  if { ! [ catch [ set y [ $errorfxn $params ] ] ] } {
    puts $of "error in $var_num"
    flush $of
    exit
  } else {
    puts $of "lset y_dels $var_num $y"
    flush $of
  }
}

# errorfxn: name of the error function that will be run
# errorPath: the path to the file containing the error function. It has be be sources
# params: the input parameters for the error function
# o_file: the path to the output file where the results from the error function will be stored
# print_str: the string that will be combined be combined with the error fucntion results and printed to the output file
proc run_errorfxn_yi { errorfxn errorPath params o_file print_str} {
  set of [open $o_file a]
  puts $of "# we are inside run_err_yi, with parameters: $params and print string: $print_str"
  
  # sourcing the error function
  puts "sourcing the error function from $errorPath"
  source $errorPath
  
  if { ! [ catch [ set y [ $errorfxn $params ] ] ] } {
    puts $of "### error with running $errorfxn ###"
    flush $of
    exit
  } else {
    puts $of "$print_str $y"
    flush $of
  }
}

# errorfxn: name of the error function that will be run
# errorPath: the path to the file containing the error function. It has be be sources
# params: the input parameters for the error function
# job_ID: the ID number of the current job
# grad_type: the type of gradient that will be calculated (forwards, backwards, or central)
proc grad { errorFxn errorProcs params job_ID grad_type } {
  #set o_file ./srun_output.tcl
  file mkdir srun_outs
  set o_file [exec mktemp -p ./srun_outs srun_out_XXXX.tcl ]  ;#consider using this to make the output files unique for earh call ofthe fucntion
  #another option is: file tempfile ?nameVar? ?template?
  set of [open $o_file a]
  set step_name "srun_step" ;# used to label and identify the job steps that will be launched during the proc
  
  puts $of "## starting $grad_type gradient calculations\n# params: $params,\tstep_name: $step_name\n# errorProcs: $errorProcs\n# errorFxn: $errorFxn"
  
  set delta 1e-8
  if { [pdbIsAvailable optimize delta] } {
    set delta [pdbGet optimize delta]
  }
  puts $of "# delta is $delta"
  
  # y0 with current parameters
  set y0_print_str "set y0"
  set flooxs_string "run_errorfxn_yi $errorFxn $errorProcs {$params} $o_file {$y0_print_str}"
  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
  #  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
  exec sh -c $sh_str &
  
  # find the y values after having changed each of the parameters
  for { set i 0 } { $i < [llength $params] } { incr i } {
    puts $of "# \tComputing partial derivative for parameter $i"
    flush $of
    
    switch $grad_type {
      forward {
        set params_delta $params
        lset params_delta $i [ expr [lindex $params $i] + $delta ]
        lappend y_dels 0
        set yi_print_str "lset y_dels $i"
        set flooxs_string "run_errorfxn_yi $errorFxn $errorProcs {$params_delta} $o_file {$yi_print_str}"
        set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        #    set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
        exec sh -c $sh_str &
      }
      backward {
        set params_delta $params
        lset params_delta $i [ expr [lindex $params $i] - $delta ]
        lappend y_dels 0
        set yi_print_str "lset y_dels $i"
        set flooxs_string "run_errorfxn_yi $errorFxn $errorProcs {$params_delta} $o_file {$yi_print_str}"
        set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        #    set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
        exec sh -c $sh_str &
        
      }
      central {
        # calculations must be done for both forward (f) and backward (b), gradient calculations
        set params_f_delta $params
        set params_b_delta $params
        lset params_f_delta $i [ expr [lindex $params $i] + $delta ]
        lset params_b_delta $i [ expr [lindex $params $i] - $delta ]
        lappend y_f_dels 0
        lappend y_b_dels 0
        set yi_f_print_str "lset y_f_dels $i"
        set yi_b_print_str "lset y_b_dels $i"
        set flooxs_f_string "run_errorfxn_yi $errorFxn $errorProcs {$params_f_delta} $o_file {$yi_f_print_str}"
        set flooxs_b_string "run_errorfxn_yi $errorFxn $errorProcs {$params_b_delta} $o_file {$yi_b_print_str}"
        set sh_f_str "echo '$flooxs_f_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        set sh_b_str "echo '$flooxs_b_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        exec sh -c $sh_f_str &
        exec sh -c $sh_b_str &
        # this could be done with only one exec if the tow sh_x_str string are combined and separated by ;, but this format is better for readability and debugging
      }
    }
  }
  
  wait_srun $job_ID $step_name $o_file
  # this switch statement can be replaced by only "source $o_file", but it is currently here for debugging
  switch $grad_type {
    forward {
      puts $of "# pre source y_dels:  $y_dels"
      source $o_file
      puts $of "# post source y_dels: $y_dels \n# y0: $y0"
    }
    backward {
      puts $of "# pre source y_dels:  $y_dels"
      source $o_file
      puts $of "# post source y_dels: $y_dels \n# y0: $y0"
    }
    central {
      puts $of "# pre source y_f_dels:  $y_f_dels"
      puts $of "# pre source y_b_dels:  $y_b_dels"
      source $o_file
      puts $of "# post source y_f_dels: $y_f_dels"
      puts $of "# post source y_b_dels: $y_b_dels \n# y0: $y0"
    }
  }
  file delete $o_file
  
  for { set i 0 } { $i < [llength $params] } { incr i } {
    switch $grad_type {
      forward {
        lappend derivs [ expr ([lindex $y_dels $i] - $y0)/($delta) ]
      }
      backward {
        lappend derivs [ expr ($y0 - [lindex $y_dels $i])/($delta) ]
      }
      central {
        lappend derivs [ expr ([lindex $y_f_dels $i] - [lindex $y_b_dels $i])/($delta) ]
      }
    }
  }
  
  puts $of "# the derivatives are: $derivs\n# the iteration error is $y0"
  flush $of
  set error $y0
  set grads $derivs
  set r_list [list $error $grads]
  return $r_list
}


proc f_grad { errorFxn errorProcs params job_ID} {
  set o_file ./srun_output.tcl
  set of [open $o_file a]
  set step_name srun_step
  
  puts $of "## starting forwards gradient calculations\n# params: $params,\tstep_name: $step_name\n# errorProcs: $errorProcs\n# errorFxn: $errorFxn"
  
  set delta 1e-8
  if { [pdbIsAvailable optimize delta] } {
    set delta [pdbGet optimize delta]
  }
  puts $of "# delta is $delta"
  
  # y0 with current parameters
  set flooxs_string "run_errorfxn_y0 $errorFxn $errorProcs {$params} $o_file none"
  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
  #  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
  exec sh -c $sh_str &
  
  # find the y values after having changed each of the parameters
  for { set i 0 } { $i < [llength $params] } { incr i } {
    puts $of "# \tComputing partial derivative for parameter $i"
    flush $of
    set params_delta $params
    lset params_delta $i [ expr [lindex $params $i] + $delta ]
    lappend y_dels 0
    
    set flooxs_string "run_errorfxn_y1 $errorFxn $errorProcs {$params_delta} $o_file $i"
    set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
    #    set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
    exec sh -c $sh_str &
  }
  
  wait_srun $job_ID $step_name $o_file
  
  puts $of "# pre source y_dels:  $y_dels"
  source $o_file
  puts $of "# post source y_dels: $y_dels \n# y0: $y0"
  #file delete $o_file
  
  for { set i 0 } { $i < [llength $params] } { incr i } {
    lappend derivs [ expr ([lindex $y_dels $i] - $y0)/($delta) ]
  }
  
  puts $of "# the derivatives are: $derivs\n# the iteration error is $y0"
  flush $of
  set error $y0
  set grads $derivs
  set r_list [list $error $grads]
  return $r_list
}

proc b_grad { errorFxn errorProcs params job_ID} {
  set o_file ./srun_output.tcl
  set of [open $o_file a]
  set step_name srun_step
  
  puts $of "## starting backwards gradient calculations\n# params: $params,\tstep_name: $step_name\n# errorProcs: $errorProcs\n# errorFxn: $errorFxn"
  
  set delta 1e-8
  if { [pdbIsAvailable optimize delta] } {
    set delta [pdbGet optimize delta]
  }
  puts $of "# delta is $delta"
  
  # y0 with current parameters
  set flooxs_string "run_errorfxn_y0 $errorFxn $errorProcs {$params} $o_file none"
  #  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
  exec sh -c $sh_str &
  
  # find the y values after having changed each of the parameters
  for { set i 0 } { $i < [llength $params] } { incr i } {
    puts $of "# \tComputing partial derivative for parameter $i"
    flush $of
    set params_delta $params
    lset params_delta $i [ expr [lindex $params $i] - $delta ]
    lappend y_dels 0
    
    set flooxs_string "run_errorfxn_y1 $errorFxn $errorProcs {$params_delta} $o_file $i"
    set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
    #set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
    exec sh -c $sh_str &
  }
  
  wait_srun $job_ID $step_name $o_file
  puts $of "# pre source y_dels:  $y_dels"
  source $o_file
  puts $of "# post source y_dels: $y_dels \n# y0: $y0"
  #file delete $o_file
  
  for { set i 0 } { $i < [llength $params] } { incr i } {
    lappend derivs [ expr ($y0 - [lindex $y_dels $i])/($delta) ]
  }
  puts $of "# the derivatives are: $derivs\n# the iteration error is $y0"
  flush $of
  set error $y0
  set grads $derivs
  set r_list [list $error $grads]
  return $r_list
}




### actual optimize procs ####
proc old_grad { errorFxn params } {
  set delta 1e-8
  if { [pdbIsAvailable optimize delta] } {
    set delta [pdbGet optimize delta]
  }
  puts "delta is $delta"
  
  # runs the error function with parameters to get y0 while also checking for errors
  # this one should be set to run in the background
  if { ! [ catch [ set y0 [ $errorFxn $params ] ] ] } {
    puts "Error in y0"
    exit
  }
  puts "the original error fucntion output is: $y0"
  
  # takes the partial derivative of each parameter and stores the derivatives in "derivs"
  #set derivs ""
  for { set i 0 } { $i < [llength $params] } { incr i } {
    puts "\tComputing partial derivative for parameter $i"
    set params_delta $params
    lset params_delta $i [ expr [lindex $params $i]+$delta ]
    # run in background
    if { ! [ catch [ set y1 [ $errorFxn $params_delta ] ] ] } {
      puts "Error in y1"
      exit
    }
    lappend derivs [ expr ($y1 - $y0)/($delta) ]
    puts "the value of derivs is: $derivs"
    #lset derivs $i [ expr ($y1 - $y0)/($delta) ]
    #puts [lindex $derivs $i ]
  }
  puts "derivs are the same as gradiant and they are: $derivs"
  return $derivs
}

# for each parameter, if the partial derivative of the error function with respect to that parameter is positive, decrease the value of the parameter, if it is negative, increase the value of the parameter
# the magnitude of the parameter change is dependent on the learning rate and if it is too grate, it is reduced to the damp value
proc updateParams { params learningRates grads damp } {
  #puts $params
  for { set i 0 } { $i < [llength $params] } { incr i } {
    #puts "lset params $i [lindex $params $i] - [lindex $learningRates $i] * [lindex $grads $i] "
    set p0 [lindex $params $i]
    puts -nonewline "\tUpdating parameter $i: $p0 - [lindex $learningRates $i] * [lindex $grads $i]"
    set up [ expr [lindex $learningRates $i] * [lindex $grads $i] ]
    if { $up > $damp } { set up $damp }
    if { $up < -$damp } { set up -$damp }
    lset params $i [ expr $p0 - $up ]
    puts " = [lindex $params $i]"
  }
  #puts $params
  #gets stdin
  return $params
}

# updates the learning rate based on how much the previous parameters changed divided by the gradient difference between the two points
proc updateLearningRates { params prevParams grads prevGrads } {
  #set learningRates ""
  for { set i 0 } { $i < [llength $params] } { incr i } {
    set gradDiff [expr [lindex $grads $i] - [lindex $prevGrads $i]]
    if { abs($gradDiff) < 1e-100 } {
      #lappend learningRates $i 1e-5
      lappend learningRates 1e-5
    } else {
      #lappend learningRates $i [ expr ([lindex $params $i] - [lindex $prevParams $i])*($gradDiff)/($gradDiff**2) ]
      lappend learningRates [ expr ([lindex $params $i] - [lindex $prevParams $i])*($gradDiff)/($gradDiff**2) ]
    }
  }
  #puts "the learning rates are: $learningRates"
  return $learningRates
}

# declares the error function, the initial input parameters, the number of iteration to be used, and the damping level
proc optimize { errorFxn params iterations errorProcs job_ID grad_type {damp 1e9} } {
  
  puts "Start Parameter Optimization"
  
  set initialStep 1e-2
  if { [pdbIsAvailable optimize initialStep] } {
    set initialStep [pdbGet optimize initialStep]
  }
  puts "the initial step is $initialStep"
  
  # get the gradient of each parameter at the current value
  #lassign [f_grad $errorFxn $errorProcs $params $job_ID] error grads  ;# forwards gradient
  #lassign [b_grad $errorFxn $errorProcs $params $job_ID] error grads  ;# backwards gradient
  lassign [grad $errorFxn $errorProcs $params $job_ID $grad_type] error grads  ;# central gradient
  
  #puts "grads have been set to: $grads"
  #puts "the error is: $error"
  flush stdout
  
  # set the learning rate of each parameter, a larger gradient gives a smaller learning rate, since small changes would have a large effect
  set learningRates ""
  for { set i 0 } { $i < [llength $params] } { incr i } {
    if { abs([lindex $grads $i]) < 1e-100 } {
      lappend learningRates 1e-5
      #lset grads $i $initialStep
    } else {
      lappend learningRates [expr $initialStep/[lindex $grads $i]]
    }
  }
  
  set grads0 $grads
  set params0 $params
  puts "Initial Parameters $params0"
  for { set i 0 } { $i < $iterations } { incr i } {
    puts "\nFor iteration #$i"
    puts "Iteration $i Grads: $grads"
    set params [updateParams $params $learningRates $grads $damp]
    #puts "Iteration $i Error: [ $errorFxn $params ]"
    puts "Iteration $i Updated Parameters $params"
    puts "Iteration $i Error: $error"
    
    #set grads [grad $errorFxn $params]
    #lassign [f_grad $errorFxn $errorProcs $params $job_ID] error grads  ;# forwards gradient
    #lassign [b_grad $errorFxn $errorProcs $params $job_ID] error grads  ;# backwards gradient
    lassign [grad $errorFxn $errorProcs $params $job_ID $grad_type] error grads  ;# central gradient
    set learningRates [updateLearningRates $params $params0 $grads $grads0]
    set params0 $params
    set grads0 $grads
  }
  set params [updateParams $params $learningRates $grads $damp]
  #puts "Final Error: [ $errorFxn $params ]"
  puts "\nFinal results"
  puts "Final Error: $error"
  puts "Final Parameters: $params"
  return $params
}

# proc optimize_while { errorFxn params } {
#
#   set learningRates ""
#   for { set i 0 } { $i < [llength $params] } { incr i } {
#     lappend learningRates 1e-15
#   }
#   set timeout 1000
#   set grads [grad $errorFxn $params]
#   set grads0 $grads
#   set params0 $params
#   while { [ $errorFxn $params ] > 1e-8 } {
# 	set params [updateParams $params $learningRates $grads]
#     set grads [grad $errorFxn $params]
# 	set learningRates [updateLearningRates $params $params0 $grads $grads0]
#     set params0 $params
# 	set grads0 $grads
# 	set timeout [ expr $timeout - 1 ]
# 	if { !$timeout } { puts timeout; break; }
#   }
#   return $params
#
# }











