
proc KandaFirstHoleConductivtyFactorTensor args {
    set name [lindex [info level 0] 0]
    lappend usage "$name stress={1 2 3 4 5 6} miller=(001)<100> mat=Silicon"
    lappend usage "Stress vector is required to be in Voigt notation string list format."
    lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
    lappend usage "Material (mat) is optional and defaults to silicon."
    set returns "Tensor list containing the relative conductivity changes for holes"
    set description {Computes the normalized conductivity change for holes according to the linear piezoresistance model}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress "$name Error: No stress vector provided"]
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }

  if {[llength $stress] != 6} {
    puts "Bad stress vector in $name"
    exit
  }
  
  set dc [ __PTypeFirstKandaTensor $miller $stress $Material ]
  
  set out ""
  lappend out $dc
  return $out
}

proc KandaFirstElecConductivtyFactorTensor args {
    set name [lindex [info level 0] 0]
    lappend usage "$name stress={1 2 3 4 5 6} miller=(001)<100> mat=Silicon"
    lappend usage "Stress vector is required to be in Voigt notation string list format."
    lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
    lappend usage "Material (mat) is optional and defaults to silicon."
    set returns "Tensor list containing the relative conductivity changes for holes"
    set description {Computes the normalized conductivity change for electrons according to the linear piezoresistance model}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress "$name Error: No stress vector provided"]
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }

  if {[llength $stress] != 6} {
    puts "Bad stress vector in $name"
    exit
  }
  
  set dc [ __NTypeFirstKandaTensor $miller $stress $Material ]
  
  set out ""
  lappend out $dc
  return $out
}

proc KandaFirstElecConductivtyFactorTensorHall args {
    set name [lindex [info level 0] 0]
    lappend usage "$name stress={1 2 3 4 5 6} mag={Bx By Bz} miller=(001)<100> mat=Silicon"
    lappend usage "Stress vector is required to be in Voigt notation string list format."
    lappend usage "Magnetic field (mag) is a vector in with x y and z components."
    lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
    lappend usage "Material (mat) is optional and defaults to silicon."
    set returns "Tensor list containing the relative conductivity changes for holes"
    set description {Computes the normalized conductivity and Hall factor changes for electrons according to the linear piezoresistance and piezo-Hall models}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress "$name Error: No stress vector provided"]
  set mag [__GetValue $args mag "$name Error: No magnetic field (mag) vector provided"]
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }

  if {[llength $stress] != 6} {
    puts "Bad stress vector in $name"
    exit
  }
  
  set dc [ __NTypeFirstKandaHallTensor $miller $stress $mag $Material ]
  
  set out ""
  lappend out $dc
  return $out
}



