proc __scalar_tensor_miller {Miller pi11 pi12 pi44} {
  set debug 0
  set pi {{ $pi11 $pi12 $pi12 0.0   0.0   0.0   }
    { $pi12 $pi11 $pi12 0.0   0.0   0.0   }
    { $pi12 $pi12 $pi11 0.0   0.0   0.0   }
    { 0.0   0.0   0.0   $pi44 0.0   0.0   }
    { 0.0   0.0   0.0   0.0   $pi44 0.0   }
  { 0.0   0.0   0.0   0.0   0.0   $pi44 }}
  for {set i 0} {$i<6} {incr i} {
    for {set j 0} {$j<6} {incr j} {
      lset pi $i $j [expr [lindex $pi $i $j]]
    }
  }
  set alpha [__approxFirstToSecondOrderRotation [ __MillerToRotation $Miller ]]
  set alphaInv [__approxFirstToSecondOrderRotation [__Transpose [ __MillerToRotation $Miller ]]]
  set pi_prime_val [__MultNxN $alpha [__MultNxN $pi $alphaInv 6] 6]
  
  if {$debug} {
    puts "Printing alpha"
    for {set i 0} {$i<6} {incr i} {
      puts "[lindex $alpha $i 0] [lindex $alpha $i 1] [lindex $alpha $i 2] [lindex $alpha $i 3] [lindex $alpha $i 4] [lindex $alpha $i 5]"
    }
    puts "Printing Pi"
    for {set i 0} {$i<6} {incr i} {
      puts "[lindex $pi_prime_val $i 0] [lindex $pi_prime_val $i 1] [lindex $pi_prime_val $i 2] [lindex $pi_prime_val $i 3] [lindex $pi_prime_val $i 4] [lindex $pi_prime_val $i 5]"
    }
  }
  return $pi_prime_val
}

proc __scalar_tensor {CrystalRotation pi11 pi12 pi44} {
  set debug 0
  set pi {{ $pi11 $pi12 $pi12 0.0   0.0   0.0   }
    { $pi12 $pi11 $pi12 0.0   0.0   0.0   }
    { $pi12 $pi12 $pi11 0.0   0.0   0.0   }
    { 0.0   0.0   0.0   $pi44 0.0   0.0   }
    { 0.0   0.0   0.0   0.0   $pi44 0.0   }
  { 0.0   0.0   0.0   0.0   0.0   $pi44 }}
  for {set i 0} {$i<6} {incr i} {
    for {set j 0} {$j<6} {incr j} {
      lset pi $i $j [expr [lindex $pi $i $j]]
    }
  }
  set alpha [ __alpha $CrystalRotation ]
  set alphaInv [ __alphaInv $CrystalRotation ]
  set pi_prime_val [__MultNxN $alpha [__MultNxN $pi $alphaInv 6] 6]
  
  if {$debug} {
    puts "Printing alpha"
    for {set i 0} {$i<6} {incr i} {
      puts "[lindex $alpha $i 0] [lindex $alpha $i 1] [lindex $alpha $i 2] [lindex $alpha $i 3] [lindex $alpha $i 4] [lindex $alpha $i 5]"
    }
    puts "Printing Pi"
    for {set i 0} {$i<6} {incr i} {
      puts "[lindex $pi_prime_val $i 0] [lindex $pi_prime_val $i 1] [lindex $pi_prime_val $i 2] [lindex $pi_prime_val $i 3] [lindex $pi_prime_val $i 4] [lindex $pi_prime_val $i 5]"
    }
  }
  return $pi_prime_val
}

proc __FirstKandaTensor { pi_mat X_vec } {
  set debug 0
  # set drho_vec [__MultAx $pi_mat $X_vec ]
  set drho_vec "0 0 0 0 0 0"
  for {set i 0} {$i<6} {incr i} {
    for {set j 0} {$j<6} {incr j} {
      lset drho_vec $i [lindex $drho_vec $i]+[lindex $pi_mat $i $j]*[lindex $X_vec $j]
    }
  }
  
  set drho_mat "{ {[lindex $drho_vec 0]} {[lindex $drho_vec 5]} {[lindex $drho_vec 4]} }
    {  {[lindex $drho_vec 5]} {[lindex $drho_vec 1]} {[lindex $drho_vec 3]} }
  {  {[lindex $drho_vec 4]} {[lindex $drho_vec 3]} {[lindex $drho_vec 2]} }"
  
  # set dsigma_mat [__MultNxN {{-1. 0 0} {0 -1. 0} {0 0 -1.}} $drho_mat  3]
  # lset dsigma_mat 0 0 [expr 1.0+[lindex $dsigma_mat 0 0]]
  # lset dsigma_mat 1 1 [expr 1.0+[lindex $dsigma_mat 1 1]]
  # lset dsigma_mat 2 2 [expr 1.0+[lindex $dsigma_mat 2 2]]
  
  set dsigma_mat {{0 0 0} {0 0 0} {0 0 0}}
  lset dsigma_mat 0 0 (1./(1.0+([lindex $drho_mat 0 0])))
  lset dsigma_mat 1 1 (1./(1.0+([lindex $drho_mat 1 1])))
  lset dsigma_mat 2 2 (1./(1.0+([lindex $drho_mat 2 2])))
  
  if {$debug} {
    for {set i 0} {$i<3} {incr i} {
      puts "[lindex $dsigma_mat $i 0] [lindex $dsigma_mat $i 1] [lindex $dsigma_mat $i 2]"
    }
    for {set i 0} {$i<3} {incr i} {
      puts "[lindex $drho_mat $i 0] [lindex $drho_mat $i 1] [lindex $drho_mat $i 2]"
    }
  }
  return $dsigma_mat
}

# pi_mat is the calculated pi coefficeint matrix X_Vect is the stress vector(xx,yy,zz,yz,xz,xy), and uB is mobility times magnetic field vector (x,y,z)
proc __FirstKandaHallTensor { pi_mat p_mat X_vec B } {
  set Bx [lindex $B 0]; set By [lindex $B 1]; set Bz [lindex $B 2];
  set debug 0
  set drho_vec [__MultAx $pi_mat $X_vec ]
  # convert the teslas to cgs
  set scal [ expr 1.0/10. ]
  
  set dR_vec [__MultAx $p_mat $X_vec ]
  
  set drho_mat {}
  lset drho_mat 0 0 [expr  1.0+[lindex $drho_vec 0]]
  lset drho_mat 0 1 [expr  $scal*$Bz*(1.0+[lindex $dR_vec 2]+[lindex $drho_vec 5])]
  lset drho_mat 0 2 [expr  $scal*$By*(1.0+[lindex $dR_vec 1]+[lindex $drho_vec 4])]
  lset drho_mat 1 0 [expr -$scal*$Bz*(1.0+[lindex $dR_vec 2]+[lindex $drho_vec 5])]
  lset drho_mat 1 1 [expr  1.0+[lindex $drho_vec 1]]
  lset drho_mat 1 2 [expr  $scal*$Bx*(1.0+[lindex $dR_vec 2]+[lindex $drho_vec 3])]
  lset drho_mat 2 0 [expr -$scal*$By*(1.0+[lindex $dR_vec 1]+[lindex $drho_vec 4])]
  lset drho_mat 2 1 [expr -$scal*$Bx*(1.0+[lindex $dR_vec 2]+[lindex $drho_vec 3])]
  lset drho_mat 2 2 [expr  1.0+[lindex $drho_vec 2]]
  set dsigma_mat [__Inv3x3 $drho_mat]
  if {$debug} {
    for {set i 0} {$i<3} {incr i} {
      puts "[lindex $dsigma_mat $i 0] [lindex $dsigma_mat $i 1] [lindex $dsigma_mat $i 2]"
    }
    for {set i 0} {$i<3} {incr i} {
      puts "[lindex $drho_mat $i 0] [lindex $drho_mat $i 1] [lindex $drho_mat $i 2]"
    }
  }
  return $dsigma_mat
}

## Silicon Specific

proc __PTypeFirstKandaTensor { Miller Stress Material } {
  
  # Piezoresistance
  if { [ pdbIsAvailable $Material ppi11  ] } { set ppi11  [ pdbGet $Material ppi11  ] } else {
    puts "Pi coefficient ppi11 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material ppi12  ] } { set ppi12  [ pdbGet $Material ppi12  ] } else {
    puts "Pi coefficient ppi12 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material ppi44  ] } { set ppi44  [ pdbGet $Material ppi44  ] } else {
    puts "Pi coefficient ppi44 not specified for material $Material"; exit;
  }
  set Pi_Matrix [__scalar_tensor_miller $Miller $ppi11 $ppi12 $ppi44]
  
  return [ __FirstKandaTensor $Pi_Matrix $Stress ]
  
}

proc __NTypeFirstKandaTensor { Miller Stress Material } {
  set Material Silicon
  # Piezoresistance
  if { [ pdbIsAvailable $Material npi11  ] } { set npi11  [ pdbGet $Material npi11  ] } else {
    puts "Pi coefficient npi11 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material npi12  ] } { set npi12  [ pdbGet $Material npi12  ] } else {
    puts "Pi coefficient npi12 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material npi44  ] } { set npi44  [ pdbGet $Material npi44  ] } else {
    puts "Pi coefficient npi44 not specified for material $Material"; exit;
  }
  set Pi_Matrix [__scalar_tensor_miller $Miller $npi11 $npi12 $npi44]
  
  return [ __FirstKandaTensor $Pi_Matrix $Stress ]
  
}

proc __NTypeFirstKandaHallTensor { Miller Stress uB Material } {
  set Material Silicon
  # Piezo Hall
  if { [ pdbIsAvailable $Material np11  ] } { set np11  [ pdbGet $Material np11  ] } else {
    puts "PiezoHall coefficient np11 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material np12  ] } { set np12  [ pdbGet $Material np12  ] } else {
    puts "PiezoHall coefficient np12 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material np44  ] } { set np44  [ pdbGet $Material np44  ] } else {
    puts "PiezoHall coefficient np44 not specified for material $Material"; exit;
  }
  set PH_Matrix [__scalar_tensor_miller $Miller $np11 $np12 $np44]
  
  # Piezoresistance
  if { [ pdbIsAvailable $Material npi11  ] } { set npi11  [ pdbGet $Material npi11  ] } else {
    puts "Pi coefficient npi11 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material npi12  ] } { set npi12  [ pdbGet $Material npi12  ] } else {
    puts "Pi coefficient npi12 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material npi44  ] } { set npi44  [ pdbGet $Material npi44  ] } else {
    puts "Pi coefficient npi44 not specified for material $Material"; exit;
  }
  set Pi_Matrix [__scalar_tensor_miller $Miller $npi11 $npi12 $npi44]
  
  return [ __FirstKandaHallTensor $Pi_Matrix $PH_Matrix $Stress $uB ]
  
}

proc __Compliance {Miller Material} {
  # Rotate the compliance matrix
  if { [ pdbIsAvailable $Material c11  ] } { set c11  [ pdbGet $Material c11  ] } else {
    puts "Compliance constant c11 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material c12  ] } { set c12  [ pdbGet $Material c12  ] } else {
    puts "Compliance constant c12 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material c44  ] } { set c44  [ pdbGet $Material c44  ] } else {
    puts "Compliance constant c44 not specified for material $Material"; exit;
  }
  return [__scalar_tensor_miller $Miller $c11 $c12 $c44]
}

proc __Stiffness {Miller Material} {
  # Rotate the compliance matrix
  if { [ pdbIsAvailable $Material c11  ] } { set c11  [ pdbGet $Material c11  ] } else {
    puts "Compliance constant c11 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material c12  ] } { set c12  [ pdbGet $Material c12  ] } else {
    puts "Compliance constant c12 not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material c44  ] } { set c44  [ pdbGet $Material c44  ] } else {
    puts "Compliance constant c44 not specified for material $Material"; exit;
  }
  set E [expr 1./$c11]
  set v [expr -$c12/$c11]
  set scal [expr $E/(1+$v)/(1-2*$v)]
  # puts [__scalar_tensor_miller $Miller [expr (1-$v)*$scal] [expr $v*$scal] [expr 1./$c44]]
  return [__scalar_tensor_miller $Miller [expr (1-$v)*$scal] [expr $v*$scal] [expr 1./$c44]]
}


