
proc __MillerToRotation {m} {
  proc format_err {m} {
    puts "Bad miller index format: given '$m' want (hkl)<uvw> format or plane orientation and in-plane orientation, where hkl and uvw correspond to xyz axies, angle brackets are used in place of square because of tcl"
    exit 
  }
  if {[string first "(" $m] == -1 && [string first ")" $m] == -1 && [string first "<" $m] == -1 && [string first ">" $m] == -1} {
    format_err $m
  }
  set plane [split $m ")"]
  set plane [string map {"(" ""} [lindex $plane 0]]
  if { [string length [string map {"-" ""} $plane]] != 3 } {
    format_err $m
  }
  set index 0
  set h ""
  if { [string index $plane $index ] == "-" } {
    incr index
    set h [expr -1*[string index $plane $index ]]
  } else {
    set h [expr [string index $plane $index ]]
  }
  incr index
  set k ""
  if { [string index $plane $index ] == "-" } {
    incr index
    set k [expr -1*[string index $plane $index ]]
  } else {
    set k [expr [string index $plane $index ]]
  }
  incr index
  set l ""
  if { [string index $plane $index ] == "-" } {
    incr index
    set l [expr -1*[string index $plane $index ]]
  } else {
    set l [expr [string index $plane $index ]]
  }
  set plane_norm [expr sqrt($h**2+$k**2+$l**2)]

  set inplane [split $m "<"]
  set inplane [string map {">" ""} [lindex $inplane 1]]
  if { [string length [string map {"-" ""} $inplane]] != 3 } {
    format_err $m
  }
  set index 0
  set u ""
  if { [string index $inplane $index ] == "-" } {
    incr index
    set u [expr -1*[string index $inplane $index ]]
  } else {
    set u [expr [string index $inplane $index ]]
  }
  incr index
  set v ""
  if { [string index $inplane $index ] == "-" } {
    incr index
    set v [expr -1*[string index $inplane $index ]]
  } else {
    set v [expr [string index $inplane $index ]]
  }
  incr index
  set w ""
  if { [string index $inplane $index ] == "-" } {
    incr index
    set w [expr -1*[string index $inplane $index ]]
  } else {
    set w [expr [string index $inplane $index ]]
  }
  set inplane_norm [expr sqrt($u**2+$v**2+$w**2)]

  if {($h*$u+$k*$v+$l*$w) > 1e-10} {
    puts "Miller indicies are not orthogonal"
    exit
  }
  
  # cross product of plane normal and inplane directions
  set c1 [expr $k*$w-$l*$v]
  set c2 [expr -$h*$w+$l*$u]
  set c3 [expr $h*$v-$u*$k]
  set c_norm [expr sqrt($c1**2+$c2**2+$c3**2)]
  set alpha {{0 0 0} {0 0 0} {0 0 0}}
  lset alpha 0 2 [expr $h/$plane_norm]
  lset alpha 1 2 [expr $k/$plane_norm]
  lset alpha 2 2 [expr $l/$plane_norm]
  lset alpha 0 0 [expr $u/$inplane_norm]
  lset alpha 1 0 [expr $v/$inplane_norm]
  lset alpha 2 0 [expr $w/$inplane_norm]
  lset alpha 0 1 [expr $c1/$c_norm]
  lset alpha 1 1 [expr $c2/$c_norm]
  lset alpha 2 1 [expr $c3/$c_norm]
  return $alpha
}
# puts [__MillerToRotation (001)<100>]
# puts [__MillerToRotation (001)<110>]
# puts [__MillerToRotation (111)<1-10>]
# puts [__MillerToRotation (111)<-110>]
# puts [__MillerToRotation (111)<0-11>]
# puts [__MillerToRotation (111)<01-1>]

proc __MillerToEulerSimple {m} {
  proc format_err {m} {
    puts "Bad miller index format: given '$m' want (hkl)<uvw> format or plane orientation and in-plane orientation"
    exit 
  }
  if {[string first "(" $m] == -1 && [string first ")" $m] == -1 && [string first "<" $m] == -1 && [string first ">" $m] == -1} {
    format_err $m
  }
  set plane [split $m ")"]
  set plane [string map {"(" ""} [lindex $plane 0]]
  if { [string length $plane] != 3 } {
    format_err $m
  }
  set h [string index $plane 0 ]
  set k [string index $plane 1 ]
  set l [string index $plane 2 ]
  set inplane [split $m "<"]
  set inplane [string map {">" ""} [lindex $inplane 1]]
  set u [string index $inplane 0 ]
  set v [string index $inplane 1 ]
  set w [string index $inplane 2 ]
  if { [string length $inplane] != 3 } {
    format_err $m
  }
  set phi1 [expr asin($w/sqrt($u**2+$v**2+$w**2) * sqrt($h**2+$k**2+$l**2)/sqrt($h**2+$k**2))]
  set Psi [expr acos($l/sqrt($l**2+$h**2+$k**2))]
  set phi2 [expr acos($k/sqrt($h**2+$k**2))]

  set phi1 [expr 180/acos(-1.)*$phi1]
  set Psi [expr 180/acos(-1.)*$Psi]
  set phi2 [expr 180/acos(-1.)*$phi2]
  return "$phi1 $Psi $phi2"
}

