
proc __GetValue {args key {err ""}} {
  set key [string tolower $key]
  set args [string map {"=" "= "} [string map {"= " "=" "=  " "=" "=   " "=" " = " "=" " =  " "=" " =   " "=" "  = " "=" "  =  " "=" "  =   " "="} $args]]
  for {set i 0} { $i < [llength $args] } { incr i } {
    set arg [lindex $args $i]
    if { [string first "=" $arg] > -1 } {
      set tmp [ split $arg "=" ]
      if { [string tolower [lindex $tmp 0]] == $key } {
        if { [lindex $tmp 1] == "" } { 
          return [lindex $args $i+1]
        } else {
          return [lindex $tmp 1]
        }
      }
    }
  }
  if { $err == "" } {
    return ""
  } else {
    puts $err
    exit
  }
}

proc __Contains {args key} {
  return [expr [ string first $key $args ] != -1]
}
proc __Remove {args key} {
  set ret ""
  foreach v $args {
    if { $key != $v } {
      lappend ret $v
    }
  }
  return $ret
}

proc __TexEscape {s} {
  set out ""
  # This is a hand implementation of the following string map command
  # [string map {"\}" "\\\}" "\{" "\\\{" "<" {\textlangle } ">" {\textrangle } "_" "\\\_"} $s]
  # Except the curly brackets don't get mapped inside an equation
  set eqn 0
  for { set i 0 } { $i < [string length $s] } { incr i } {
    set c [string index $s $i]
    if { $c == "<" } {
      set out [string cat $out {\textlangle }]
    } elseif { $c == ">" } {
      set out [string cat $out {\textrangle }]
    } elseif { !$eqn && $c == "_" } {
      set out [string cat $out {\_}]
    } elseif { $c == "$" } {
      set eqn [expr !$eqn]
      set out "$out$c"
    } elseif { !$eqn && $c == "\}" } {
      set out [string cat $out "\\\}"]
    } elseif { !$eqn && $c == "\{" } {
      set out [string cat $out "\\\{"]
    } else {
      set out "$out$c"
    }
  }
  return $out
}

proc __Help {args name usage returns {description ""}} {
  if { [__Contains $args -helptex] } {
    # tex special character escape
    set name [string map {"_" "\\\_"} $name]
    set returns [__TexEscape $returns]
    set description [__TexEscape $description]
    puts "\\\subsubsection{$name} "
    puts "\\\paragraph{Usage}"
    foreach u $usage {
      puts "\\par [__TexEscape $u]"
    }
    puts "\\\paragraph{Returns} $returns"
    if { $description != "" } {
      puts "\\\paragraph{Description}"
      puts "$description"
    }
    return 1
  } elseif { [__Contains $args -help] } {
    # puts "\u001b\[1m\u001b\[33m$name help:\u001b\[0m"
    puts "$name help"
    puts	-nonewline "Usage:"
    foreach u $usage {
      puts "\t$u"
    }
    puts "Returns: $returns"
    if { $description != "" } {
      puts "Description: $description"
    }
    return 1
  } else {
    return 0
  }
}

proc Test args {
  lappend usage "Test -help"
  if { [__Help $args [lindex [info level 0] 0] $usage "nothing" "Simple test proc to test documentation"] } {
    return 1
  }
  return 0
}








