
# takes an sqlite3 database command
# and database column parameter list 
# returns a dictionary of key parameters and values list
proc getParams {db} {
  set params ""
  set tmp ""
  $db eval "SELECT Parameter FROM param_type WHERE type='pro_param';" values {
    lappend tmp $values(Parameter)
  }
  foreach p $tmp {
    set col "COUNT(DISTINCT $p)"
    $db eval "SELECT $col FROM process ;" values {
      if {$values($col) > 1 } {
        lappend params $p
      }
    }
  }

  set tmp ""
  $db eval "SELECT Parameter FROM param_type WHERE type='dev_param';" values {
    lappend tmp $values(Parameter)
  }
  foreach p $tmp {
    set col "COUNT(DISTINCT $p)"
    $db eval "SELECT $col FROM device ;" values {
      if {$values($col) > 1 } {
        lappend params $p
      }
    }
  }

  return $params
}

proc getDataNames {db} {
  set tmp ""
  $db eval "SELECT Parameter FROM param_type WHERE type='process_output' OR type='device_output' OR type='pro_data' OR type='dev_data';" values {
    lappend tmp $values(Parameter)
  }
  return $tmp
}

proc getParamsRaw {db table} {
  set tmp ""
  #$db eval "SELECT sql FROM sqlite_master WHERE tbl_name = '$table' AND type = 'table'" values 
  $db eval "SELECT name FROM PRAGMA_TABLE_INFO('$table');" values {
    lappend tmp [lindex [array get values] 1]
  }
  return $tmp
}

# test if table exists
proc tableExists {db table} {
  set exists 0
  $db eval "SELECT name FROM sqlite_master WHERE type='table'" values {
    if { $values(name) == $table } { set exists 1 }
  }
  return $exists
} 

# takes an sqlite3 database command
# and database column parameter list 
# returns a dictionary of key parameters and values list
proc getParamValsFromDB {db table params} {
  foreach param $params {
    set tmp ""
    $db eval "SELECT DISTINCT $param FROM $table ORDER BY $param ASC " values {
      if { $values($param) != "" } {
    	#puts $values($param)
    	lappend tmp $values($param)
      }
    }
    dict set paramVals $param $tmp
  }
  return $paramVals
}

# takes an sqlite3 database command
# and database column parameter list 
# returns a dictionary of key parameters and values list
proc getParamValsFromDBCount {db table params} {
  set paramVals ""
  foreach {param vals} $params {
    set tmp ""
    foreach val $vals {
      $db eval "SELECT COUNT($param) FROM $table WHERE $param='$val'" values {
      		#puts [dict get [array get values] "COUNT($param)"]
      		lappend tmp [dict get [array get values] "COUNT($param)"]
      }
    }
    dict set paramVals $param $tmp
  }
  return $paramVals
}

# gathers database histogram data from a key value dictionary of parameter value
# also takes a column name "var" to gather the data for (example real current)
# returns a list "$npts $min $max d" which is the number of histogram data points
# the min of the data and max of the data and the plmatrix name
proc dbDataToPldata {db table paramVal var state} {
  set where "1"
  foreach {param val} $paramVal {
    set where "$where AND $param = $val"
  }
  $db eval "SELECT MAX($var), MIN($var), COUNT($var) FROM $table
  	WHERE $where " values {
  		#puts [array names values]
  		set min [lindex [array get values "MIN($var)"] 1]
  		set max [lindex [array get values "MAX($var)"] 1]
  		set npts [lindex [array get values "COUNT($var)"] 1]
  }
  if { !$npts } {
    return "0 0 0 null"
  }
  set scal [OrderScale $max]
  set unit [OrderUnit $max]
  set max [expr $max*$scal]
  set min [expr $min*$scal]
  if { ! [info exists tkstate::matnum] } {
    set tkstate::matnum 0
  } else {
    set tkstate::matnum [expr $tkstate::matnum+1]
  }
  set mat d$tkstate::matnum
  matrix -persist $mat f $npts
  set data ""
  $db eval "SELECT $var FROM $table WHERE $where" values {
        lappend data [expr [lindex [array get values "$var"] 1]*$scal]
  }
  for { set i 0 } { $i < [llength $data] } { incr i } { 
        $mat $i = [lindex $data $i]
  }
  if {$state != ""} {
    set stats [stats list= $data]
    set $state\histmean [Round [dict get $stats avg] 10000]
    set $state\histsd [Round [dict get $stats sd] 10000]
    set $state\histskew [Round [dict get $stats skew] 10000]
    set $state\histkurt [Round [dict get $stats exkurt] 10000]
    set $state\histmin [Round $min 10000]
    set $state\histmax [Round $max 10000]
  }
  return "$npts $min $max $mat $unit"
}
proc Round {val scal} { return [expr round($scal*$val)/(1.0*$scal)] }
# number needed to scale a value to a single order of magnitude
proc OrderScale {val} { 
  if { [expr round($val*1e-18)] } { return 1e-18 
  } elseif { [expr round($val*1e-15)] } { return 1e-15
  } elseif { [expr round($val*1e-12)] } { return 1e-12
  } elseif { [expr round($val*1e-9)] } { return 1e-9
  } elseif { [expr round($val*1e-6)] } { return 1e-6
  } elseif { [expr round($val*1e-3)] } { return 1e-3
  } elseif { [expr round($val)] } { return 1
  } elseif { [expr round($val*1e3)] } { return 1e3
  } elseif { [expr round($val*1e6)] } { return 1e6
  } elseif { [expr round($val*1e9)] } { return 1e9
  } elseif { [expr round($val*1e12)] } { return 1e12
  } elseif { [expr round($val*1e15)] } { return 1e15
  } elseif { [expr round($val*1e18)] } { return 1e18
  } else {return 1e21}
}
# unit needed for the value
proc OrderUnit {val} { 
  if { [expr round($val*1e-18)] } { return "E"
  } elseif { [expr round($val*1e-15)] } { return "P"
  } elseif { [expr round($val*1e-12)] } { return "T"
  } elseif { [expr round($val*1e-9)] } { return "G"
  } elseif { [expr round($val*1e-6)] } { return "M"
  } elseif { [expr round($val*1e-3)] } { return "k"
  } elseif { [expr round($val)] } { return ""
  } elseif { [expr round($val*1e3)] } { return "m"
  } elseif { [expr round($val*1e6)] } { return "u"
  } elseif { [expr round($val*1e9)] } { return "n"
  } elseif { [expr round($val*1e12)] } { return "p"
  } elseif { [expr round($val*1e15)] } { return "f"
  } elseif { [expr round($val*1e18)] } { return "a"
  } else {return "too small" }
}


