
proc plhistpng {tmp geom histdata {xlab "#frConductance (S)"} {ylab "#frFrequency"} {title ""}} {
  lassign $histdata npts min max plmatrix

  set w loopback

  plsdev pngcairo
  plsfam 0 0 1000000
  # plspage 0 0 0 0 100 100
  plsetopt -geometry $geom
  #plsetopt -mar 0.1
  plsfnam $tmp
  plscolbg 255 255 255
  plinit
  plschr 0 1.5
  $w cmd plcol1 0.5
  $w cmd plhist $npts $plmatrix $min $max 44 0
  $plmatrix delete ;# needed to clean up the matrix allocated by persist
  plschr 0 1.8
  #$w cmd pllab $xlab $ylab $title
  $w cmd plmtex b 2.5 0.5 0.5 $xlab 
  $w cmd plmtex l 3.5 0.5 0.5 $ylab 
  $w cmd plmtex t 1. 0.5 0.5 $title
  plschr 0 1.
  plend
}

proc plhistCanvasImage { geom canvas paramVal state data database} {
  set IsRawData [tableExists $database data]
  if { $IsRawData } {
    set histdata [dbDataToPldata $database data $paramVal $data $state]
  } else {
    set histdata [dbDataToPldata $database "device JOIN process ON process.structure = device.structure" $paramVal $data $state]
  }
  $canvas dchars 2 0 1000
  set unitName  [set $state\unit] 
  set unitAbrev [set $state\unitAbrev]
  if { ! [lindex $histdata 0] } { 
    $canvas insert 2 0 "No Data"
    # image delete histpng
    image create bitmap $state\histpng -background "white"
  } else {
    set unit [lindex $histdata 4]
    set tmp /tmp/[file tempfile].png
    plhistpng $tmp $geom $histdata "#fr$unitName ($unit$unitAbrev)"
    image create photo $state\histpng -file $tmp
    file delete $tmp
  }
}

proc __histupdate { param vals state toplvl dataName database ind } {
  set $state\selected_$param [lindex $vals $ind ]
  set width [set $state\width ]
  set height [set $state\height ]
  dict set $state\hist_cur_paramval $param [lindex $vals $ind ]
  plhistCanvasImage $width\x$height $toplvl.win.canvas [set $state\hist_cur_paramval] $state $dataName $database
}
proc histSlider { param paramVals state toplvl dataName database } {
  # ind has to be last since it is set by the slider
  frame $toplvl.param$param
  set vals [dict get $paramVals $param]
  set ind 0
  set $state\selected_$param [lindex $vals $ind ]
  pack [label $toplvl.param$param.label -text $param] -side left 
  pack [label $toplvl.param$param.label2 -textvariable $state\selected_$param] -side left 
  pack [scale $toplvl.param$param.scale -from 0 -to [expr [llength $vals]-1] -digits 0 -resolution 1 -orien horizontal -length 400 -command "__histupdate $param {$vals} $state $toplvl $dataName $database " -showvalue 0 ] -side right
  pack $toplvl.param$param -fill x -padx 40 -pady 5
}
  
proc resizeHistWidget {toplvl dataName database} {
  set state tkstate::[string map {. _} $toplvl]
  set height [set $state\height]
  set width [set $state\width]
  $toplvl.win.canvas configure -width $width -height $height 
  $toplvl.win.canvas coords 2 [expr $width*0.5] [expr $height*0.5]  
  plhistCanvasImage $width\x$height $toplvl.win.canvas [set $state\hist_cur_paramval] $state $dataName $database
}

proc histWidget {paramVals dataName database unit unitAbrev} {
  if { ! [info exists tkstate::histnum] } {
    set tkstate::histnum 0
  } else {
    set tkstate::histnum [expr $tkstate::histnum+1]
  }
  set histnum $tkstate::histnum
  set toplvl .hist$histnum
  toplevel $toplvl
  set width 500
  set state tkstate::[string map {. _} $toplvl]
  set $state\initial 1
  set $state\lastRefresh 0
  set $state\unit $unit 
  set $state\unitAbrev $unitAbrev
  set curParamVal $state\hist_cur_paramval
  set $curParamVal ""
  foreach {param val} $paramVals {
    set $curParamVal "[set $curParamVal] $param [lindex $val 0]"
  }
  set height [expr 600-50*[llength [dict keys $paramVals]]]
  set $state\sliderHeight [expr 50*[llength [dict keys $paramVals]]]
  set $state\height $height
  set $state\width $width
  
  frame $toplvl.win 
  pack $toplvl.win 
  pack [canvas $toplvl.win.canvas -width $width -height $height -background white] -side left
  pack [frame $toplvl.win.stats -padx 20 -pady 20 ] -side right
  pack [label $toplvl.win.stats.title -text "Stats"] 

  set $state\histmean 0
  set $state\histsd 0
  set $state\histskew 0
  set $state\histkurt 0
  set $state\histmin 0
  set $state\histmax 0

  pack [frame $toplvl.win.stats.min  ]
  pack [label $toplvl.win.stats.min.title -width 16 -anchor w -text "Min"] -side left 
  pack [label $toplvl.win.stats.min.value -width 10 -anchor e -textvariable $state\histmin] -side right

  pack [frame $toplvl.win.stats.max  ]
  pack [label $toplvl.win.stats.max.title -width 16 -anchor w -text "Max"] -side left 
  pack [label $toplvl.win.stats.max.value -width 10 -anchor e -textvariable $state\histmax] -side right

  pack [frame $toplvl.win.stats.mean  ]
  pack [label $toplvl.win.stats.mean.title -width 16 -anchor w -text "Mean"] -side left 
  pack [label $toplvl.win.stats.mean.value -width 10 -anchor e -textvariable $state\histmean] -side right

  pack [frame $toplvl.win.stats.sd ]
  pack [label $toplvl.win.stats.sd.title -width 16 -anchor w -text "Standard Deviation"] -side left 
  pack [label $toplvl.win.stats.sd.value -width 10 -anchor e -textvariable $state\histsd] -side right

  pack [frame $toplvl.win.stats.skew ]
  pack [label $toplvl.win.stats.skew.title -width 16 -anchor w -text "Skewness"] -side left 
  pack [label $toplvl.win.stats.skew.value -width 10 -anchor e -textvariable $state\histskew] -side right

  pack [frame $toplvl.win.stats.kurt ]
  pack [label $toplvl.win.stats.kurt.title -width 16 -anchor w -text "Excess Kurtosis"] -side left 
  pack [label $toplvl.win.stats.kurt.value -width 10 -anchor e -textvariable $state\histkurt] -side right

  plhistCanvasImage $width\x$height $toplvl.win.canvas [set $curParamVal] $state $dataName $database
  $toplvl.win.canvas create image 0 0 -anchor nw -image $state\histpng
  $toplvl.win.canvas create text [expr $width*0.5] [expr $height*0.5] 
  
  foreach param [dict keys $paramVals] {
    if { [llength [dict get $paramVals $param]] < 2 } { continue; }
    histSlider $param $paramVals $state $toplvl $dataName $database 
  }

  set $state\dataName $dataName
  set $state\database $database
  bind $toplvl <Configure> {
    if {"%W" eq [winfo toplevel %W]} {
      set toplvl "%W"
      set state tkstate::[string map {. _} $toplvl]
      set cur_width [set $state\width]
      set w "%w"
      set w [expr $w]
      set h "%h"
#puts "$w $h"
      # only resize the window if we have moved past the initial size before
      if { !(($w == 836) && [set $state\initial]) && 
            (([clock milliseconds]-[set $state\lastRefresh]) > 20) } {
        set $state\initial 0
        set $state\lastRefresh [clock milliseconds]
        set sliderHeight [set $state\sliderHeight ]
        set h [expr $h-$sliderHeight]
        if { [expr $w*0.75] > $h } {
          if {abs([expr $h/0.75]-$cur_width) > 20} {
            set $state\height [expr round($h)]
            set $state\width [expr round($h/0.75)]
            resizeHistWidget $toplvl [set $state\dataName] [set $state\database] 
          }
        } else {
          if {abs($w-$cur_width) > 20} {
            set $state\height [expr round($w*0.75)]
            set $state\width [expr round($w)]
            resizeHistWidget $toplvl [set $state\dataName] [set $state\database] 
          }
        }
      }
    }
  }
}

proc histSelectWidget { db_name } {
  if { ! [info exists tkstate::dbnum] } {
    set tkstate::dbnum 0
  } else {
    set tkstate::dbnum [expr $tkstate::dbnum+1]
  }
  set database db$tkstate::dbnum
  set toplvl .$database
  set state tkstate::[string map {. _} $toplvl]
  toplevel $toplvl
  sqlite3 $database $db_name
  set IsRawData [tableExists $database data]
  if { $IsRawData } {
    set params [getParamsRaw $database data]
    set dataNames $params
  } else {
    set params [getParams $database]
    set dataNames [getDataNames $database]
  }
  pack [frame $toplvl.paramselect -width 500] -side left
  pack [label $toplvl.paramselect.title -text "Select Input Parameters"] 
  foreach param $params {
    pack [checkbutton $toplvl.paramselect.p$param -variable $state\p$param -text $param]
  }
  pack [frame $toplvl.data -width 500] -side left
  pack [label $toplvl.data.title -text "Select Histogram Data"] 

  set $state\data [lindex $params 0]
  foreach param $dataNames {
    pack [radiobutton $toplvl.data.d$param -variable $state\data -text $param -value $param]
  }

  set $state\unit Conductance
  set $state\unitAbrev S
  pack [frame $toplvl.unit ] 
  pack [label $toplvl.unit.title -text "Data Unit Name"] -side left
  pack [entry $toplvl.unit.entry -width 20 -background white -relief ridge -borderwidth 8 -textvariable $state\unit] -side left
  pack [frame $toplvl.unitAbrev ] 
  pack [label $toplvl.unitAbrev.title -text "Data Unit Abbreviation"] -side left
  pack [entry $toplvl.unitAbrev.entry -width 15 -background white -relief ridge -borderwidth 8 -textvariable $state\unitAbrev] -side left
  pack [button $toplvl.plot -font {Helvetica 18 bold} -height 5 -width 20 -text "Plot Histogram" -command "histplot $state {$params} $database"] -side left

  proc histplot { state params database } {
    set newParams ""
    foreach param $params {
      if { [set $state\p$param ] } {
        lappend newParams $param
      }
    }
    if { ![llength $newParams] } { return }
    # example of how to set default values for future implimentation
    # set defaults [getParamValsFromDB $database data "mu sd"]
    # set paramVals "[getParamValsFromDB $database data $newParams] $defaults" 
    set IsRawData [tableExists $database data]
    if { $IsRawData } {
      set paramVals [getParamValsFromDB $database data $newParams]
    } else {
      set paramVals [getParamValsFromDB $database "device JOIN process ON process.structure = device.structure" $newParams]
    }
    puts "histWidget $paramVals [set $state\data] $database"
    histWidget $paramVals [set $state\data] $database [set $state\unit] [set $state\unitAbrev]
  }

}

