
proc plscatpng {tmp geom scatdatax scatdatay {xlab "#frConductance (S)"} {ylab "#frFrequency"} {title ""}} {
  lassign $scatdatax npts xmin xmax x
  lassign $scatdatay npts ymin ymax y

  set w loopback

  plsdev pngcairo
  plsfam 0 0 1000000
  plsetopt -geometry $geom
  plsfnam $tmp
  plscolbg 250 250 250
  plinit
  $w cmd plcol1 0.5

  $w cmd pladv 0
  $w cmd plvpor 0.15 0.85 0.1 0.9
  $w cmd plwind $xmin $xmax $ymin $ymax
  $w cmd plbox "bnstv" 0.0 0 "bnstv" 0.0 0
  $w cmd plstring $npts $x $y "."

  $x delete ;# needed to clean up the matrix allocated by persist
  $y delete ;# needed to clean up the matrix allocated by persist
  $w cmd pllab $xlab $ylab $title
  plend
}

proc plscatCanvasImage { geom canvas paramVal state} {
  set database [set $state\database] 
  set x [set $state\x] 
  set y [set $state\y]
  set IsRawData [tableExists $database data]
  if { $IsRawData } {
    set table data
  } else {
    set table "device JOIN process ON process.structure = device.structure"
  }
  set scatdatax [dbDataToPldata $database $table $paramVal $x ""]
  set scatdatay [dbDataToPldata $database $table $paramVal $y ""]

  lassign $scatdatax npts __ ___ x
  lassign $scatdatay npts __ ___ y
  set sumx 0
  set sumy 0
  for { set i 0 } { $i < $npts } { incr i } {
    set xx [$x $i]
    set yy [$y $i]
    set sumx [expr $sumx+$xx]
    set sumy [expr $sumy+$yy]
  }
  set sdx 0
  set sdy 0
  set covxy 0
  set avgx [expr $sumx/$npts]
  set avgy [expr $sumy/$npts]
  for { set i 0 } { $i < $npts } { incr i } {
    set xx [$x $i]
    set yy [$y $i]
    set sdx [expr $sdx+($xx-$avgx)**2]
    set sdy [expr $sdy+($yy-$avgy)**2]
    set covxy [expr $covxy+($xx-$avgx)*($yy-$avgy)]
  }
  set sdx [expr sqrt($sdx/$npts)]
  set sdy [expr sqrt($sdy/$npts)]
  set covxy [expr $covxy/$npts]
  set $state\scatcov [Round [expr $covxy] 10000]
  if { abs($sdx) > 1e-50 && abs($sdy) > 1e-50 } {
    set $state\scatcor [Round [expr $covxy/($sdx*$sdy)] 10000]
    set $state\scatr2 [Round [expr ($covxy/($sdx*$sdy))**2] 10000]
  }

  set xunitName  [set $state\xunit] 
  set xunitAbrev [set $state\xunitAbrev]
  set yunitName  [set $state\yunit] 
  set yunitAbrev [set $state\yunitAbrev]
  $canvas dchars 2 0 1000
  if { (! [lindex $scatdatax 0]) && (! [lindex $scatdatay 0] ) } { 
    $canvas insert 2 0 "No Data"
    # image delete scatpng
    image create bitmap $state\scatpng -background "white"
  } else {
    set xunit [lindex $scatdatax 4]
    set yunit [lindex $scatdatay 4]
    set tmp /tmp/[file tempfile].png
    plscatpng $tmp $geom $scatdatax $scatdatay "#fr$xunitName ($xunit$xunitAbrev)" "#fr$yunitName ($yunit$yunitAbrev)"
    image create photo $state\scatpng -file $tmp
    file delete $tmp
  }
}
  
proc __scatupdate { param vals state toplvl ind } {
  set $state\selected_$param [lindex $vals $ind ]
  set width [set $state\width ]
  set height [set $state\height ]
  dict set $state\scat_cur_paramval $param [lindex $vals $ind ]
  plscatCanvasImage $width\x$height $toplvl.win.canvas [set $state\scat_cur_paramval] $state
}
proc scatSlider { param paramVals state toplvl } {
  # ind has to be last since it is set by the slider
  frame $toplvl.param$param
  set vals [dict get $paramVals $param]
  set ind 0
  set $state\selected_$param [lindex $vals $ind ]
  pack [label $toplvl.param$param.label -text $param] -side left 
  pack [label $toplvl.param$param.label2 -textvariable $state\selected_$param] -side left 
  pack [scale $toplvl.param$param.scale -from 0 -to [expr [llength $vals]-1] -digits 0 -resolution 1 -orien horizontal -length 400 -command "__scatupdate $param {$vals} $state $toplvl " -showvalue 0 ] -side right
  pack $toplvl.param$param -fill x -padx 40 -pady 5
}

proc scatWidget {paramVals x y database state xunit xunitAbrev yunit yunitAbrev} {
  if { ! [info exists tkstate::scatnum] } {
    set tkstate::scatnum 0
  } else {
    set tkstate::scatnum [expr $tkstate::scatnum+1]
  }
  set scatnum $tkstate::scatnum
  set toplvl .scat$scatnum
  toplevel $toplvl
  set width 500
  set state tkstate::[string map {. _} $toplvl]
  set $state\initial 1
  set $state\lastRefresh 0
  set $state\database $database
  set $state\x $x
  set $state\y $y
  set $state\xunit $xunit 
  set $state\xunitAbrev $xunitAbrev
  set $state\yunit  $yunit 
  set $state\yunitAbrev $yunitAbrev
  set curParamVal $state\scat_cur_paramval
  set $curParamVal ""
  foreach {param val} $paramVals {
    set $curParamVal "[set $curParamVal] $param [lindex $val 0]"
  }
  set height [expr 600-50*[llength [dict keys $paramVals]]]
  set $state\sliderHeight [expr 50*[llength [dict keys $paramVals]]]
  set $state\height $height
  set $state\width $width
  
  frame $toplvl.win 
  pack $toplvl.win 
  pack [canvas $toplvl.win.canvas -width $width -height $height -background white] -side left
  pack [frame $toplvl.win.stats -padx 20 -pady 20 ] -side right
  pack [label $toplvl.win.stats.title -text "Stats"] 

  set $state\scatcov 0
  pack [frame $toplvl.win.stats.cov  ]
  pack [label $toplvl.win.stats.cov.title -width 16 -anchor w -text "Covariance"] -side left 
  pack [label $toplvl.win.stats.cov.value -width 10 -anchor e -textvariable $state\scatcov] -side right

  set $state\scatcor 0
  pack [frame $toplvl.win.stats.cor  ]
  pack [label $toplvl.win.stats.cor.title -width 16 -anchor w -text "Correlation (r)"] -side left 
  pack [label $toplvl.win.stats.cor.value -width 10 -anchor e -textvariable $state\scatcor] -side right

  set $state\scatr2 0
  pack [frame $toplvl.win.stats.r2  ]
  pack [label $toplvl.win.stats.r2.title -width 16 -anchor w -text "r^2"] -side left 
  pack [label $toplvl.win.stats.r2.value -width 10 -anchor e -textvariable $state\scatr2] -side right

  plscatCanvasImage $width\x$height $toplvl.win.canvas [set $curParamVal] $state
  $toplvl.win.canvas create image 0 0 -anchor nw -image $state\scatpng
  $toplvl.win.canvas create text [expr $width*0.5] [expr $height*0.5] 
  
  foreach param [dict keys $paramVals] {
    if { [llength [dict get $paramVals $param]] < 2 } { continue; }
    scatSlider $param $paramVals $state $toplvl 
  }

  proc resizeScatWidget {toplvl} {
    set state tkstate::[string map {. _} $toplvl]
    set height [set $state\height]
    set width [set $state\width]
    $toplvl.win.canvas configure -width $width -height $height 
    $toplvl.win.canvas coords 2 [expr $width*0.5] [expr $height*0.5]  
    plscatCanvasImage $width\x$height $toplvl.win.canvas [set $state\scat_cur_paramval] $state
  }
  
  bind $toplvl <Configure> {
    if {"%W" eq [winfo toplevel %W]} {
      set toplvl "%W"
      set state tkstate::[string map {. _} $toplvl]
      set cur_width [set $state\width]
      set w "%w"
      set w [expr $w]
      set h "%h"
#puts "$w $h"
      # only resize the window if we have moved past the initial size before
      if { !(($w == 836) && [set $state\initial]) && 
            (([clock milliseconds]-[set $state\lastRefresh]) > 20) } {
        set $state\initial 0
        set $state\lastRefresh [clock milliseconds]
        set sliderHeight [set $state\sliderHeight ]
        set h [expr $h-$sliderHeight]
        if { [expr $w*0.75] > $h } {
          if {abs([expr $h/0.75]-$cur_width) > 20} {
            set $state\height [expr round($h)]
            set $state\width [expr round($h/0.75)]
            resizeScatWidget $toplvl 
          }
        } else {
          if {abs($w-$cur_width) > 20} {
            set $state\height [expr round($w*0.75)]
            set $state\width [expr round($w)]
            resizeScatWidget $toplvl 
          }
        }
      }
    }
  }
}

proc scatSelectWidget { db_name } {
  if { ! [info exists tkstate::dbnum] } {
    set tkstate::dbnum 0
  } else {
    set tkstate::dbnum [expr $tkstate::dbnum+1]
  }
  set database db$tkstate::dbnum
  set toplvl .$database
  set state tkstate::[string map {. _} $toplvl]
  toplevel $toplvl
  sqlite3 $database $db_name
  set IsRawData [tableExists $database data]
  if { $IsRawData } {
    set params [getParamsRaw $database data]
    set dataNames $params
  } else {
    set params [getParams $database]
    set dataNames [getDataNames $database]
  }
  pack [frame $toplvl.paramselect -width 500] -side left
  pack [label $toplvl.paramselect.title -text "Select Input Parameters"] 
  foreach param $params {
    pack [checkbutton $toplvl.paramselect.p$param -variable $state\p$param -text $param]
  }

  pack [frame $toplvl.xdata -width 500] -side left
  pack [label $toplvl.xdata.title -text "Select Scatter X Data "] 
  set $state\xdata [lindex $params 0]
  foreach param $dataNames {
    pack [radiobutton $toplvl.xdata.d$param -variable $state\xdata -text $param -value $param]
  }
  pack [frame $toplvl.ydata -width 500] -side left
  pack [label $toplvl.ydata.title -text "Select Scatter Y Data "] 
  set $state\ydata [lindex $params 0]
  foreach param $dataNames {
    pack [radiobutton $toplvl.ydata.d$param -variable $state\ydata -text $param -value $param]
  }

  #X xunit widget
  set $state\xunit Conductance
  set $state\xunitAbrev S
  pack [frame $toplvl.xunit ] 
  pack [label $toplvl.xunit.title -text "X Data unit Name"] -side left
  pack [entry $toplvl.xunit.entry -width 20 -background white -relief ridge -borderwidth 8 -textvariable $state\xunit] -side left
  pack [frame $toplvl.xunitAbrev ] 
  pack [label $toplvl.xunitAbrev.title -text "X Data unit Abbreviation"] -side left
  pack [entry $toplvl.xunitAbrev.entry -width 15 -background white -relief ridge -borderwidth 8 -textvariable $state\xunitAbrev] -side left
  
  #Y unit widget
  set $state\yunit Capacitance
  set $state\yunitAbrev F
  pack [frame $toplvl.yunit ] 
  pack [label $toplvl.yunit.title -text "Y Data Unit Name"] -side left
  pack [entry $toplvl.yunit.entry -width 20 -background white -relief ridge -borderwidth 8 -textvariable $state\yunit] -side left
  pack [frame $toplvl.yunitAbrev ] 
  pack [label $toplvl.yunitAbrev.title -text "Y Data Unit Abbreviation"] -side left
  pack [entry $toplvl.yunitAbrev.entry -width 15 -background white -relief ridge -borderwidth 8 -textvariable $state\yunitAbrev] -side left

  pack [button $toplvl.plot -font {Helvetica 14 bold} -height 5 -width 20 -text "Scatter Plot" -command "scatplot $state {$params} $database"] -side left

  proc scatplot { state params database } {
    set newParams ""
    foreach param $params {
      if { [set $state\p$param ] } {
        lappend newParams $param
      }
    }
    if { ![llength $newParams] } { return }
    # example of how to set default values for future implimentation
    # set defaults [getParamValsFromDB $database data "mu sd"]
    # set paramVals "[getParamValsFromDB $database data $newParams] $defaults" 
    set IsRawData [tableExists $database data]
    if { $IsRawData } {
      set paramVals [getParamValsFromDB $database data $newParams]
    } else {
      set paramVals [getParamValsFromDB $database "device JOIN process ON process.structure = device.structure" $newParams]
    }
    puts "scatWidget $paramVals [set $state\xdata] $database"
    scatWidget $paramVals [set $state\xdata] [set $state\ydata] $database $state [set $state\xunit] [set $state\xunitAbrev] [set $state\yunit] [set $state\yunitAbrev]
  
  }

}

