#*************************************************************************
#*									 *
#*     Copyright c 2002 The board of regents of the University of 	 *
#*                      Florida.  All rights reserved.			 *
#*     This subroutine may not be used outside of the FLOO* computer	 *
#*     programs without the prior written consent of the University of 	 *
#*     Florida.								 *
#*									 *
#*************************************************************************/



namespace eval tkstate {}
  proc getOpenFileHelper { {count 0} } {
    font configure TkDefaultFont -size 11
    if { $count > 2 } {
      # probably not linux
      return
    }
    if { ! [info exists ::tk::dialog::file::__tk_filedialog] } {
      after 100 ::uiutils::getOpenFileHelper [incr count]
      return
    }
    $::tk::dialog::file::__tk_filedialog(dirMenu) configure -font myDefaultFont
    $::tk::dialog::file::__tk_filedialog(ent) configure -font myDefaultFont
    $::tk::dialog::file::__tk_filedialog(typeMenu) configure -font myDefaultFont
    $::tk::dialog::file::__tk_filedialog(icons) configure -font myDefaultFont
  }

proc __PlotGrid {state} {
  fs name=$state select

  pdbSet plot fontScale 1.1
  window pngcairo file=tmp[set state].png width= 500 height= 500
  plot2d grid
  window null
  image create bitmap [set state]png -background "white"
  image create photo [set state]png -file tmp[set state].png
  file delete tmp[set state].png
}

proc __PlotData {state} {
  fs name=$state select

  set materials "*(0"
  foreach m [mater list] {
    if { [set $state$m] } {
      set materials "$materials+Material($m)"
    }
  }
  set materials "$materials)"
  proc log {state} {
    set data [set [set state]data]
    if { [set [set state]log] } {
      return "($data>0?log10($data):-log10(-$data))"
    } else { return $data }
  }
  pdbSet plot fontScale 1.1
  window pngcairo file=tmp[set state].png width= 500 height= 500
  sel z= [log $state]$materials
  plot2d levels= 100 colorbar title= [set [set state]data]
  window null
  image create bitmap [set state]png -background "white"
  image create photo [set state]png -file tmp[set state].png
  file delete tmp[set state].png
}

proc __StructureGUI { fil } {
  if { ! [info exists tkstate::structguinum] } {
    set tkstate::structguinum 0
  } else {
    set tkstate::structguinum [expr $tkstate::structguinum+1]
  }
  set structguinum $tkstate::structguinum
  set toplvl .structgui[set structguinum]
  set state tkstate::[string map {. _} $toplvl]
  fs name=$state add

  fs name=$state select
  struct infile = $fil

  __PlotGrid $state

  toplevel $toplvl -width 800 -height 500
  pack [canvas $toplvl.plot -width 500 -height 500 -background white] -side left
  $toplvl.plot create image 0 0 -anchor nw -image [set state]png

  pack [frame $toplvl.options -width 300 -padx 10 -pady 10] -side left
  pack [button $toplvl.options.plotGrid -padx 5 -pady 5 -font {Helvetica 10 bold} -height 0 -width 10 -text "Plot Grid" -command "__PlotGrid $state"] 

  pack [frame $toplvl.options.data -padx 10 -pady 10]
  set choices [sel list]
  set [set state]data [lindex $choices 0]
  # set len [llength $choices]
  set len 5
  pack [listbox $toplvl.options.data.list -height $len ]
  bind $toplvl.options.data.list {<<ListboxSelect>>}  "set [set state]data \[$toplvl.options.data.list get active\]"
  $toplvl.options.data.list insert 0 {*}$choices
  pack [label $toplvl.options.data.list_choice -pady 1 -textvariable [set state]data]

  pack [label $toplvl.options.data.materials -pady 5 -width 16 -anchor w -text "Materials"]
  foreach m [mater list] {
    set $state$m 1
    pack [checkbutton $toplvl.options.data.m$m -text "$m" -variable $state$m]
  }

  set [set state]log 0
  pack [label $toplvl.options.data.log_label -width 16 -anchor w -text "Options"]
  pack [checkbutton $toplvl.options.data.log -text "Log Scale" -variable [set state]log]

  # pack [frame $toplvl.options.data.xmin -width 300 -padx 10 -pady 10]
  # pack [label $toplvl.options.data.xmin.label -anchor w -text "xmin"] -side left
  # pack [entry $toplvl.options.data.xmin.value -width 10 -textvariable [set state]xmin] -side left

  pack [button $toplvl.options.data.plotData -padx 5 -pady 5 -font {Helvetica 10 bold} -height 0 -width 10 -text "Plot Data" -command "__PlotData $state"] 
  
}

proc BuildFileMenu { file } {

    #set a global for file manipulation
    global curFile
    set curFile temp.fl

    menu $file -tearoff 0
    $file add command -label Open -command {
	  global curFile
      after idle ::getOpenFileHelper
	  set file [tk_getOpenFile -filetypes {{{Database Files} {.db .fdb}} {{Structure Files} {.str}}} -title "Read a Floops Structure File Or Parameter DB"]
	  if { [string first ".str" $file] > -1 } {
          __StructureGUI $file
	  } elseif { [string first ".fdb" $file] > -1 || [string first ".db" $file] > -1 } {
          histSelectWidget $file
          scatSelectWidget $file
	  } else {
          puts "unrecognized file"
      }
    }

    # $file add command -label "Quick 1D Grid" -command {
	# Grid1D
    # }

    # $file add command -label "1D Grid Dialog" -command {
	# GridDialog1D
    # }

    # $file add separator
    # $file add command -label Save -command {
	# global curFile
	# struct outf = $curFile
    # }

    # $file add command -label "Save As" -command {
	# global curFile
	# set file [tk_getSaveFile -title "Write a Floops Structure File"]
	# if { [string compare $file ""] != 0 } {
	#     struct outf = $file
	#     set curFile $file
	# }
    # }
    $file add separator
    $file add command -label Quit -command {exit}
}

proc BuildPostMenu { post } {
    menu $post -tearoff 0
    $post configure -postcommand {
	if { [flps_grid qual dim] != 0 } {
	    sel z = $Select
	}
	DimensionChangeMenu
    }

    # #set the display section
    # Text input section
    # text .txtdisp -width 80 -height 20 -wrap word
    # pack .txtdisp -side bottom -expand 1 -fill x

    #add a display for the current selected plot variable
    frame .seldr -bd 2
    pack .seldr -side top -fill x -anchor w
    label .seldr.l1 -text "Current Plot Variable: "
    label .seldr.l2 -textvariable Select
    pack .seldr.l1 .seldr.l2 -side left -fill x -anchor w

    $post add command -label "Plot 1D" -state disabled -command {
	Plot1Dialog
    }
    $post add command -label "Plot Materials" -state disabled -command {
	plot.2d bound
    }
    $post add command -label "Plot Grid" -state disabled -command {
	plot.2d grid
    }
    $post add command -label "Contour" -state disabled -command {
	contour -gui
    }
    $post add separator

    $post add command -label "Interface" -command {
	catch ".txtdisp delete 0.0 end"
	.txtdisp insert end [interface -gui]
    }
    $post add command -label "Interpolate" -command {
	catch ".txtdisp delete 0.0 end"
	.layerstxtdisp insert end [interpolate -gui]
    }
    $post add command -label "Layers" -command {
	catch ".txtdisp delete 0.0 end"
	if { [flps_grid qual dim] == 1 } {
	    .txtdisp insert end [layers]
	} else {
	    .txtdisp insert end [layers -gui]
	}
    }
}
