proc ClusterEqn {Mat Sol} {
    set pM [pdbName $Mat]

    #the smic Vacancy capture rate
    set KfV {[DiffLimit Silicon Vac 0.0]}
    set KfV2 {[DiffLimit Silicon V2 0.0]}

    #capture of vacancies by larger clusters
    term name = SmicVac add $pM eqn = "$KfV*Vac*Smic"
    term name = SmicV2 add $pM eqn = "$KfV2*V2*Smic"

    #capture of interstitials by clusters
    set KfI [pdbDelayDouble $pM Smic KfI]
    set BSmic [pdbDelayDouble $pM Smic Bind]
    term name = SmicI add $pM eqn = "$KfI*Smic*(Int-$BSmic)"

    #how fast do we grow out of the Smic's
    set KNI [pdbDelayDouble Silicon C311 KnI2]
    set Ci2star [pdbDelayDouble Silicon I2 Cstar]
    term name=D311NI add $pM eqn = "$KNI*Smic*I2*log(I2/$Ci2star)"
    set NSize 9

    #set the smic equation
    pdbSetString $pM Smic Equation "ddt(Smic) + SmicVac + 2.0*SmicV2 - SmicI + $NSize*D311NI"

    #subtract the capture of I's
    set de [pdbGetString $pM Int Equation]
    pdbSetString $pM Int Equation "$de + SmicI"

    #subtract the capture of I2's
    set de [pdbGetString $pM I2 Equation]
    pdbSetString $pM I2 Equation "$de + D311NI"

    #subtract the capture of Vac's
    set de [pdbGetString $pM Vac Equation]
    pdbSetString $pM Vac Equation "$de + SmicVac"

    #subtract the capture of V2's
    set de [pdbGetString $pM V2 Equation]
    pdbSetString $pM V2 Equation "$de + SmicV2"

    #Get the 311 Nucleation 
    set de [pdbGetString $pM D311 Equation]
    pdbSetString $pM D311 Equation "$de - D311NI"

    #Get the 311 Nucleation 
    set de [pdbGetString $pM C311 Equation]
    pdbSetString $pM C311 Equation "$de - ($NSize+2.0)*D311NI"
}


proc 311Eqn {Mat Sol} {
    set pM [pdbName $Mat]

    #compute the parameters for the forward reaction with interstitials
    set BindI [pdbDelayDouble Silicon C311 BindI]
    set KfI [pdbDelayDouble Silicon C311 KfI]
    set KfI2 [pdbDelayDouble Silicon C311 KfI2]
    #add a term for the formation / dissolution of Ints's off 311's
    term add name = C311FormI eqn = "$KfI * D311 * 14.0 * (Int - $BindI)"

    set KfI2 [pdbDelayDouble Silicon C311 KfI2]
    term add name = C311FormI2 eqn = "$KfI2 * D311 * 14.0 * I2"

    #Get the spontaneous combustion rate
    set DRate [pdbDelayDouble Silicon C311 Decay]
    term add name = ISize eqn = "D311 / (C311 + 1.0e-1)"
    term add name = DRate eqn = $DRate

    #do the defect equation
    set de [pdbGetString Silicon D311 Equation]
    pdbSetString Silicon D311 Equation "$de + ddt(D311) + D311 * DRate + $KfI * D311 * 14.0 * $BindI * ISize"

    #create the C311 equation
    set de [pdbGetString Silicon C311 Equation]
    pdbSetString $pM $Sol Equation "$de + ddt(C311) - C311FormI - 2.0*C311FormI2 + C311 * DRate"

    #subtract the capture of I's
    set de [pdbGetString $pM Int Equation]
    pdbSetString $pM Int Equation "$de + C311FormI - C311 * DRate"

    #subtract the capture of I2's
    set de [pdbGetString $pM I2 Equation]
    pdbSetString $pM I2 Equation "$de + C311FormI2"
}


proc C311Init { Mat Sol } {
    set pdbMat [pdbName $Mat]
    pdbSetString $pdbMat C311 Equation ""
    pdbSetString $pdbMat D311 Equation ""
}


