#*************************************************************************
#*									 *
#*     Copyright c 1998 The board of regents of the University of 	 *
#*                      Florida.  All rights reserved.			 *
#*     This subroutine may not be used outside of the FLOO* computer	 *
#*     programs without the prior written consent of the University of 	 *
#*     Florida.								 *
#*									 *
#*************************************************************************/
#*   AxisMenu.tcl                Version 1.1     */
#*   Last Modification : 01/23/98 08:52:50  */

proc LogFormat {graph value} {
    set exp [expr log10($value)]
    return 10^[string range $exp 0 [expr [string wordstart $exp 3] - 1]]
}

proc AxisDialog {graph axis} {
    toplevel .ad
    table .ad
    wm title .ad "Axis Options for $axis"

    #title the dialog box
    label .ad.title -text $axis 

    #allow choice of axis name and font
    set AxisName [$graph axis cget $axis -title]
    label .ad.l1 -text "Axis Name:"
    entry .ad.e1 -width 20
    .ad.e1 delete 0 end
    .ad.e1 insert 0 $AxisName
    global NameFont
    set NameFont [$graph axis cget $axis -titlefont]
    button .ad.ba -text "Name Font" -font $NameFont -command {
	set NameFont [SelectFont "Axis Name" $NameFont]
	.ad.ba configure -font $NameFont
    }

    table .ad \
	.ad.title 0,0 -cspan 3 -anchor w	\
	.ad.l1 1,0 -anchor w		\
	.ad.e1 1,1			\
	.ad.ba 1,2

    frame .ad.sep1 -relief sunken -height 2 -bd 2
    table .ad .ad.sep1 2,0 -cspan 3 -fill x -pady 4

    #get current limits and scaling
    global min max logscale loose
    if {[$graph axis cget $axis -log]} {
	set logscale log
    } else {
	set logscale linear
    }
    if {[$graph axis cget $axis -loose]} {
	set loose Loose
    } else {
	set loose Exact
    }
    if {[$graph axis cget $axis -descending]} {
	set Direction Decreasing
    } else {
	set Direction Increasing
    }
    set min [$graph axis cget $axis -min]
    set max [$graph axis cget $axis -max]
    ToggleButton .ad.b1 logscale log linear
    ToggleButton .ad.b2 loose Loose Exact
    ToggleButton .ad.b3 Direction Increasing Decreasing
    label .ad.mnl1 -text "Minimum:"
    entry .ad.mne1 -width 20
    .ad.mne1 delete 0 end
    .ad.mne1 insert 0 $min
    label .ad.mxl1 -text "Maximum:"
    entry .ad.mxe1 -width 20
    .ad.mxe1 delete 0 end
    .ad.mxe1 insert 0 $max
    button .ad.mnmx -text "AutoScale" -command {
	.ad.mxe1 delete 0 end
	.ad.mne1 delete 0 end
    }

    table .ad \
	.ad.b1 3,0 -padx 5 	\
	.ad.b2 3,1 -padx 5 	\
	.ad.b3 3,2 -padx 5 -fill x 	\
	.ad.mnl1 4,0 -padx 5	\
	.ad.mne1 4,1 -padx 5	\
	.ad.mnmx 4,2 -padx 5 -fill x	\
	.ad.mxl1 5,0 -padx 5	\
	.ad.mxe1 5,1 -padx 5	
    
    frame .ad.sep2 -relief sunken -height 2 -bd 2
    table .ad .ad.sep2 6,0 -cspan 3 -fill x -pady 4

    #display the axis, font for tick marks
    global Show TickFont
    if {[$graph axis cget $axis -hide]} {
	set Show Hide
    } else {
	set Show Display
    }
    ToggleButton .ad.db Show Display Hide
    set TickFont [$graph axis cget $axis -tickfont]
    button .ad.btf -text "Tick Font" -font $TickFont -command {
	set TickFont [SelectFont "Tick Marks" $TickFont]
	.ad.btf configure -font $TickFont
    }

    table .ad \
	.ad.db 7,0 \
	.ad.btf 7,1

    button .ad.c1 -text Cancel -command {destroy .ad}
    button .ad.c2 -text Update -command {
	global graph symbol Show logscale loose TickFont Direction
	set Name [.ad.e1 get]
	set min [.ad.mne1 get]
	set max [.ad.mxe1 get]
	set axis [.ad.title cget -text]
	$graph axis configure $axis -title $Name
	$graph axis configure $axis -titlefont $NameFont
	set lg [expr [string compare $logscale log]==0]
	set ls [expr [string compare $loose Loose]==0]
	set dir [expr [string compare $Direction Decreasing]==0]
	$graph axis configure $axis -loose $ls
	$graph axis configure $axis -log $lg
	$graph axis configure $axis -descending $dir
	if {$lg} {
	    $graph axis configure $axis -command LogFormat
	} else {
	    $graph axis configure $axis -command ""
	}
	$graph axis configure $axis -min $min
	$graph axis configure $axis -max $max
	$graph axis configure $axis -tickfont $TickFont
	set h [expr [string compare $Show Hide]==0]
	$graph axis configure $axis -hide $h
	destroy .ad
    }
    frame .ad.sep -relief sunken -width 2 -bd 2 

    table .ad \
	.ad.sep 0,3 -rspan 8 -fill y -padx 5	\
	.ad.c1 1,4 -anchor e	\
	.ad.c2 2,4 -anchor e \

    grab .ad
    tkwait window .ad
}


